UpgradeList = LibStub("AceAddon-3.0"):NewAddon("UpgradeList", "AceConsole-3.0", "AceEvent-3.0", "AceHook-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("UpgradeList")

local UPGRADELIST_VERSION = GetAddOnMetadata('UpgradeList', 'Version')

local LSM = LibStub("LibSharedMedia-3.0")
local DEBUGMODE = false

local function debugUL(message, flag)
	if flag then
		DEFAULT_CHAT_FRAME:AddMessage("|cFFE08800Upgrade List DEBUG: |r " .. message)
	end
end

-- Register some media
LSM:Register("sound", "Dungeon Ready", [[Sound\Interface\levelup2.wav]])
LSM:Register("sound", "Level Up", [[Sound\Interface\LevelUp.wav]])
LSM:Register("sound", "Capture Flag", [[Sound\Interface\PVPFlagCapturedHordeMono.wav]])
LSM:Register("sound", "Horde Flag Taken", [[Sound\Interface\PVPFlagTakenHordeMono.wav]])
LSM:Register("sound", "Alliance Flag Taken", [[Sound\Interface\PVPFlagTakenMono.wav]])
LSM:Register("sound", "Ready?", [[Sound\Interface\ReadyCheck.wav]])
LSM:Register("sound", "Whip", [[Sound\Item\Weapons\Whip\BullWhipHit1.wav]])
LSM:Register("sound", "Puppy Bark", [[Sound\Emitters\Emitter_Dalaran_Petstore_Puppy_07.wav]])
LSM:Register("sound", "Squire Horn", [[Sound\Events\squire_horn_aa.wav]])
LSM:Register("sound", "Zelda-Heart Piece", [[Interface\Addons\UpgradeList\Sounds\LTTP_Get_HeartPiece.ogg]]) 
LSM:Register("sound", "Zelda-Item", [[Interface\Addons\UpgradeList\Sounds\LTTP_ItemFanfare_Stereo.ogg]]) 
LSM:Register("sound", "Zelda-Rupee", [[Interface\Addons\UpgradeList\Sounds\LTTP_Rupee1.ogg]]) 
LSM:Register("sound", "Zelda-Secret", [[Interface\Addons\UpgradeList\Sounds\LTTP_Secret.ogg]]) 


local displaychoices = {
	chatwindow = "Chat Window",
	scrollwindow = "Message Window",
}

local autodeletechoices = {
	[1] = "Assume acquired item is for Primary Spec list",
	[2] = "Assume acquired item is for Secondary Spec list",
	[3] = "Do nothing",
	[4] = "Ask me which spec the acquired item is for",
}

local onehandweaponchoices = {
	[1] = "Assume acquired weapon is for Main Hand list",
	[2] = "Assume acquired weapon is for Off Hand list",
	[3] = "Do nothing",
	[4] = "Ask me which list the acquired weapon is for",
}

local WeaponAndArmorSubTypes = {
	Bows = "59320",
	Crossbows = "63532",
	Daggers = "63536",
	Guns = "60210",
	Fishing_Poles = "19970",
	Fist_Weapons = "63538",
	One_Handed_Axes = "59443",
	One_Handed_Maces = "59459",
	One_Handed_Swords = "63533",
	Polearms = "64810",
	Staves = "59525",
	Thrown = "68608",
	Two_Handed_Axes = "59330",
	Two_Handed_Maces = "59492",
	Two_Handed_Swords = "61346",
	Wands = "59460",
	Miscellaneous = "5956",
	Cloth = "59349",
	Leather = "69579",
	Mail = "59504",
	Plate = "59316",
	Shields = "55069",
	Relics = "64674",
}

local EquipLocationNumbers = {
	{"INVTYPE_HEAD", 1, "Head"},
	{"INVTYPE_NECK", 2, "Neck"},
	{"INVTYPE_SHOULDER", 3, "Shoulder"},
	{"INVTYPE_CLOAK", 4, "Back"},
	{"INVTYPE_CHEST", 5, "Chest"},
	{"INVTYPE_ROBE", 5, "Chest"},
	{"INVTYPE_WRIST", 6, "Wrist"},
	{"INVTYPE_HAND", 7, "Hand"},
	{"INVTYPE_WAIST", 8, "Waist"},
	{"INVTYPE_LEGS", 9, "Leg"},
	{"INVTYPE_FEET", 10, "Foot"},
	{"INVTYPE_FINGER", 11, "Ring"},
	{"INVTYPE_TRINKET", 12, "Trinket"},
	{"INVTYPE_WEAPONMAINHAND", 13, "Main"},
	{"INVTYPE_2HWEAPON", 13, "Main"},
	{"INVTYPE_WEAPONOFFHAND", 14, "Off"},
	{"INVTYPE_HOLDABLE", 14, "Off"},
	{"INVTYPE_SHILED", 14, "Off"},
	{"INVTYPE_RANGED", 15, "Ranged"},
	{"INVTYPE_THROWN", 15, "Ranged"},
	{"INVTYPE_RANGEDRIGHT", 15, "Ranged"},
	{"INVTYPE_RELIC", 15, "Ranged"},
} --Everything except INVTYPE_WEAPON, INVTYPE_BAG, INVTYPE_TABARD, and INVTYPE_BODY

local TabNames = {
	{1, "Head", "Head"},
	{2, "Neck", "Neck"},
	{3, "Shoulder", "Shoulders"},
	{4, "Back", "Back"},
	{5, "Chest", "Chest"},
	{6, "Wrist", "Wrists"},
	{7, "Hand", "Hands"},
	{8, "Waist", "Waist"},
	{9, "Leg", "Legs"},
	{10, "Foot", "Feet"},
	{11, "Ring", "Rings"},
	{12, "Trinket", "Trinkets"},
	{13, "Main", "Main Hand"},
	{14, "Off", "Off Hand"},
	{15, "Ranged", "Ranged"},
	{16, "Other", "Other"},
}

local waitTable = {};
local waitFrame = nil;

local function UpgradeList_wait(delay, func, ...)
  if(type(delay)~="number" or type(func)~="function") then
    return false;
  end
  if(waitFrame == nil) then
    waitFrame = CreateFrame("Frame","WaitFrame", UIParent);
    waitFrame:SetScript("onUpdate",function (self,elapse)
      local count = #waitTable;
      local i = 1;
      while(i<=count) do
        local waitRecord = tremove(waitTable,i);
        local d = tremove(waitRecord,1);
        local f = tremove(waitRecord,1);
        local p = tremove(waitRecord,1);
        if(d>elapse) then
          tinsert(waitTable,i,{d-elapse,f,p});
          i = i + 1;
        else
          count = count - 1;
          f(unpack(p));
        end
      end
    end);
  end
  tinsert(waitTable,{delay,func,{...}});
  return true;
end

local localizedItemType = {}

for itemType,itemNumber in pairs(WeaponAndArmorSubTypes) do
	_, _, _, _, _, _, localizedItemType[itemType] = GetItemInfo(tonumber(itemNumber));
end

local EquippableItems = {
	DEATHKNIGHT = {"Cloth", "Leather", "Mail", "Plate", "Relics", "One_Handed_Maces", "Two_Handed_Maces", "Polearms", "One_Handed_Swords", "Two_Handed_Swords", "One_Handed_Axes", "Two_Handed_Axes", "Fishing_Poles", "Miscellaneous"},
	DRUID = {"Cloth", "Leather", "Relics", "Daggers", "Fist_Weapons", "One_Handed_Maces", "Polearms", "Staves", "Two_Handed_Maces", "Fishing_Poles", "Miscellaneous"},
	HUNTER = {"Cloth", "Leather", "Mail", "Guns", "Bows", "Crossbows", "Thrown", "Fist_Weapons", "One_Handed_Axes", "One_Handed_Swords", "Polearms", "Staves", "Two_Handed_Axes", "Two_Handed_Swords", "Fishing_Poles", "Miscellaneous"},
	MAGE = {"Cloth", "Staves", "Wands", "Daggers", "One_Handed_Swords", "Fishing_Poles", "Miscellaneous"},
	PALADIN = {"Cloth", "Leather", "Mail", "Plate", "Shields", "Relics", "One_Handed_Maces", "Two_Handed_Maces", "Polearms", "One_Handed_Swords", "Two_Handed_Swords", "One_Handed_Axes", "Two_Handed_Axes", "Fishing_Poles", "Miscellaneous"},
	PRIEST = {"Cloth", "Staves", "Wands", "Daggers", "One_Handed_Maces", "Fishing_Poles", "Miscellaneous"},
	ROGUE = {"Cloth", "Leather", "Daggers", "Fist_Weapons", "One_Handed_Maces", "One_Handed_Axes", "One_Handed_Swords", "Bows", "Crossbows", "Thrown", "Guns", "Fishing_Poles", "Miscellaneous"},
	SHAMAN = {"Cloth", "Leather", "Mail", "Shields", "Daggers", "Fist_Weapons", "One_Handed_Maces", "One_Handed_Axes", "Relics", "Staves", "Two_Handed_Maces", "Two_Handed_Axes", "Fishing_Poles", "Miscellaneous"},
	WARLOCK = {"Cloth", "Staves", "Wands", "Daggers", "One_Handed_Swords", "Fishing_Poles", "Miscellaneous"},
	WARRIOR = {"Cloth", "Leather", "Mail", "Plate", "Shields", "Bows", "Crossbows", "Fist_Weapons", "Guns", "Thrown", "One_Handed_Axes", "One_Handed_Maces", "One_Handed_Swords", "Polearms", "Staves", "Two_Handed_Axes", "Two_Handed_Maces", "Two_Handed_Swords", "Fishing_Poles", "Miscellaneous"},
}

local options = {
	name = "Upgrade List",
	handler = UpgradeList,
	type = 'group',
	args = {
		togglebutton = { 
			type = "execute",
			name = "Open Upgrade List",
			order = 1,
			desc = "This button will Open the Upgrade List addon.",
			func = "OpenUpgradeList",
		},
		lockframe = { --option is UpgradeListDBVar.profile.mainframe.lock
			type = "toggle",
			name = "Lock",
			desc = "Lock the Upgrade List frame.",
			order = 2,
			get = "MainFrameMovableGet",
			set = "MainFrameMovableSet",
		},
		minimapbutton = { --option is UpgradeListDBVar.profile.minimap.hide
			type = "toggle",
			name = "Hide Minimap Button",
			order = 3,
			get = "MinimapButtonGet",
			set = "MinimapButtonSet",
		},
		scale = { --option is UpgradeListDBVar.profile.mainframe.scale
			type = "range",
			name = "Scale",
			min = 0.01,
			max = 2,
			bigStep = 0.01,
			isPercent = true,
			get = "ScaleGet",
			set = "ScaleSet",
			order = 4,
			width = "full",
		},
		header = {
			name = "Detected Upgrade Options",
			type = "header",
			order = 5,
		},
		alertmessagepotential = { --option is UpgradeListDBVar.profile.messages.potential
			name = "Enable Messages",
			type = "toggle",
			desc = "Enable to allow Upgrade List to send you a message if an item on your list is detected.",
			order = 6,
			get = "AlertMessagePotentialGet",
			set = "AlertMessagePotentialSet",
		},
		alertmessagespotentialdropdown = { --option is UpgradeListDBVar.profile.messages.potentiallocation
			name = "Message Location:",
			type = "select",
			desc = "Select where you would like Upgrade List to display messages about detected upgrades.",
			values = displaychoices,
			get = "AlertMessagePotentialDropdownGet",
			set = "AlertMessagePotentialDropdownSet",
			style = "dropdown",
			order = 7,
			disabled = "DisableMessagePotentialDropdown",
		},
		alertsoundpotential = { --option is UpgradeListDBVar.profile.sound.potential
			name = "Enable Sound",
			type = "toggle",
			desc = "Enable to allow Upgrade List to play a sound if an item on your list is detected.",
			order = 8,
			get = "AlertSoundPotentialGet",
			set = "AlertSoundPotentialSet",
		},
		alertsoundpotentialdropdown = { --option is UpgradeListDBVar.profile.sound.potentialchoice
			name = "Sound:",
			type = "select",
			dialogControl = "LSM30_Sound",
			desc = "Select the sound you want to play when an item on your list is detected.",
			values = AceGUIWidgetLSMlists.sound,
			get = "AlertSoundPotentialDropdownGet",
			set = "AlertSoundPotentialDropdownSet",
			style = "dropdown",
			order = 9,
			disabled = "DisableSoundPotentialDropdown",
		},
		header2 = {
			name = "Acquired Upgrade Options",
			type = "header",
			order = 10,
		},
		alertmessageacquired = { --option is UpgradeListDBVar.profile.messages.acquired
			name = "Enable Messages",
			type = "toggle",
			desc = "Enable to allow Upgrade List to send you a message if an item on your list is acquired.",
			order = 11,
			get = "AlertMessageAcquiredGet",
			set = "AlertMessageAcquiredSet",
		},
		alertmessagesacquireddropdown = { --option is UpgradeListDBVar.profile.messages.acquiredlocation
			name = "Message Location:",
			type = "select",
			desc = "Select where you would like Upgrade List to display messages about acquired upgrades.",
			values = displaychoices,
			get = "AlertMessageAcquiredDropdownGet",
			set = "AlertMessageAcquiredDropdownSet",
			style = "dropdown",
			order = 12,
			disabled = "DisableMessageAcquiredDropdown",
		},
		alertsoundacquired = { --option is UpgradeListDBVar.profile.sound.acquired
			name = "Enable Sound",
			type = "toggle",
			desc = "Enable to allow Upgrade List to play a sound if an item on your list is acquired.",
			order = 13,
			get = "AlertSoundAcquiredGet",
			set = "AlertSoundAcquiredSet",
		},
		alertsoundacquireddropdown = { --option is UpgradeListDBVar.profile.sound.acquiredchoice
			name = "Sound:",
			type = "select",
			dialogControl = "LSM30_Sound",
			desc = "Select the sound you want to play when an item on your list is acquired.",
			values = AceGUIWidgetLSMlists.sound,
			get = "AlertSoundAcquiredDropdownGet",
			set = "AlertSoundAcquiredDropdownSet",
			style = "dropdown",
			order = 14,
			disabled = "DisableSoundAcquiredDropdown",
		},
		header3 = {
			name = "Auto-Delete Options",
			type = "header",
			order = 15,
		},
		autodelete = { --option is UpgradeListDBVar.profile.autodelete.enable
			name = "Enable Auto-Delete",
			type = "toggle",
			desc = "Enable to allow Upgrade List to auto-delete acquired items along with items of a lower priority (excluding items in the \"Other\" tab).",
			order = 16,
			get = "AutoDeleteGet",
			set = "AutoDeleteSet",
			width = "full",
		},
		autodeletemessage = { --option is UpgradeListDBVar.profile.autodelete.messages
			name = "Enable Messages",
			type = "toggle",
			desc = "Enable to allow Upgrade List to send you a message when it auto-deletes items from your list.",
			order = 17,
			get = "AutoDeleteMessageGet",
			set = "AutoDeleteMessageSet",
			disabled = "DisableAutoDeleteMessage",
		},
		autodeletemessagedropdown = { --option is UpgradeListDBVar.profile.autodelete.messagelocation
			name = "Message Location:",
			type = "select",
			desc = "Select where you would like Upgrade List to display messages about auto-deletion.",
			values = displaychoices,
			get = "AutoDeleteMessageDropdownGet",
			set = "AutoDeleteMessageDropdownSet",
			style = "dropdown",
			order = 18,
			disabled = "DisableAutoDeleteMessageDropdown",
		},
		ignorerings = {
			name = "Ignore Rings",
			type = "toggle",
			desc = "Check this option to make Upgrade List ignore Rings when auto-deleting.",
			order = 19,
			get = "IgnoreRingsGet",
			set = "IgnoreRingsSet",
			disabled = "DisableIgnoreRings",
		},
		ignoretrinkets = {
			name = "Ignore Trinkets",
			type = "toggle",
			desc = "Check this option to make Upgrade List ignore Trinkets when auto-deleting.",
			order = 20,
			get = "IgnoreTrinketsGet",
			set = "IgnoreTrinketsSet",
			disabled = "DisableIgnoreTrinkets",
		},
		autodeletedescription = {
			name = "When auto-deleting, if an item is on both the primary spec list and secondary spec list:",
			order = 21,
			type = "description",
			width = "double",
			fontSize = "medium",
		},
		autodeletechoicedropdown = { --option is UpgradeListDBVar.profile.autodelete.choice
			name = "Select action:",
			type = "select",
			desc = "Select how you would like Upgrade List to handle items in both spec lists.",
			values = autodeletechoices,
			get = "AutoDeleteChoiceDropdownGet",
			set = "AutoDeleteChoiceDropdownSet",
			style = "radio",
			order = 22,
			width = "double",
			disabled = "DisableAutoDeleteChoiceDropdown",
		},
		header4 = {
			name = "One-Handed Weapon Options",
			type = "header",
			order = 23,
		},
		onehandweapondescription = {
			name = "When you receive a 1-handed weapon, if it is on your Main Hand list and your Off Hand list:",
			order = 24,
			type = "description",
			width = "double",
			fontSize = "medium",
		},
		onehandweaponchoicedropdown = {
			name = "Select action:",
			type = "select",
			desc = "Select how you would like Upgrade List to handle 1-handed weapons in both main hand and off hand lists.",
			values = onehandweaponchoices,
			get = "OneHandWeaponChoiceDropdownGet",
			set = "OneHandWeaponChoiceDropdownSet",
			style = "radio",
			order = 25,
			width = "double",
		},
		header5 = {
			name = "Database Options",
			type = "header",
			order = 26,
		},
		deletebutton = { 
			type = "execute",
			name = "Delete ALL Data",
			order = 27,
			desc = "Delete ALL DATA in all tabs in both specs?",
			func = "ResetDatabase",
			confirm = true,
		},
	},
}

local defaults = { 
	char = {
		PrimarySpecList = {
			Head = {
				Row = { },
				Count = 1,
			},
			Neck = {
				Row = {	},
				Count = 1,
			},
			Shoulder = {
				Row = {	},
				Count = 1,
			},
			Back = {
				Row = {	},
				Count = 1,
			},
			Chest = {
				Row = {	},
				Count = 1,
			},
			Wrist = {
				Row = {	},
				Count = 1,
			},
			Hand = {
				Row = {	},
				Count = 1,
			},
			Waist = {
				Row = {	},
				Count = 1,
			},
			Leg = {
				Row = {	},
				Count = 1,
			},
			Foot = {
				Row = {	},
				Count = 1,
			},
			Ring = {
				Row = {	},
				Count = 1,
			},
			Trinket = {
				Row = {	},
				Count = 1,
			},
			Main = {
				Row = {	},
				Count = 1,
			},
			Off = {
				Row = {	},
				Count = 1,
			},
			Ranged = {
				Row = {	},
				Count = 1,
			},
			Other = {
				Row = {	},
				Count = 1,
			},
		},
		SecondarySpecList = {
			Head = {
				Row = {	},
				Count = 1,
			},
			Neck = {
				Row = {	},
				Count = 1,
			},
			Shoulder = {
				Row = {	},
				Count = 1,
			},
			Back = {
				Row = {	},
				Count = 1,
			},
			Chest = {
				Row = {	},
				Count = 1,
			},
			Wrist = {
				Row = {	},
				Count = 1,
			},
			Hand = {
				Row = {	},
				Count = 1,
			},
			Waist = {
				Row = {	},
				Count = 1,
			},
			Leg = {
				Row = {	},
				Count = 1,
			},
			Foot = {
				Row = {	},
				Count = 1,
			},
			Ring = {
				Row = {	},
				Count = 1,
			},
			Trinket = {
				Row = {	},
				Count = 1,
			},
			Main = {
				Row = {	},
				Count = 1,
			},
			Off = {
				Row = {	},
				Count = 1,
			},
			Ranged = {
				Row = {	},
				Count = 1,
			},
			Other = {
				Row = {	},
				Count = 1,
			},
		},
		SavedOptions = {
			PrimaryCheck = nil,
			SecondaryCheck = nil,
			SelectedInstance = nil,
			SelectedBoss = nil,
			SelectedDifficulty = nil,
			tableUL = nil,
		},
	},
	profile = {
		ignoreflags = {
			rings = false,
			trinkets = false,
		},
		minimap = {
			hide = false,
		},
		mainframe = {
			lock = false,
			scale = 1.0,
			showoptions = true,
		},
		messages = {
			potential = true,
			potentiallocation = "scrollwindow",
			acquired = true,
			acquiredlocation = "scrollwindow",
			scrollwindow = nil,
		},
		sound = {
			potential = true,
			potentialchoice = "None",
			acquired = true,
			acquiredchoice = "None",
		},
		autodelete = {
			enable = true,
			messages = true,
			messagelocation = "chatwindow",
			choice = 1,
		},
		onehandweapon = {
			choice = 1,
		},
	},
}

UpgradeListLDB = LibStub("LibDataBroker-1.1"):NewDataObject("UpgradeList", {
	type = "launcher",
	label = "Upgrade List",
	icon = "Interface\\Icons\\Achievement_BG_returnXflags_def_WSG",
})


function UpgradeListLDB.OnClick(self, button)
	if (button == "LeftButton") then
		UpgradeList:OpenUpgradeList()
	else
		InterfaceOptionsFrame_OpenToCategory(UpgradeList.optionsFrame)
	end
end

local icon = LibStub("LibDBIcon-1.0")

function UpgradeList:OnInitialize()
	UpgradeListDBVar = LibStub("AceDB-3.0"):New("UpgradeListDB", defaults, "Default")
	LibStub("AceConfig-3.0"):RegisterOptionsTable("UpgradeList", options)
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("UpgradeList", "Upgrade List")
	--LibStub("AceConfig-3.0"):RegisterOptionsTable("UpgradeList-Profiles", LibStub("AceDBOptions-3.0"):GetOptionsTable(UpgradeListDBVar))
	--self.profilesFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("UpgradeList-Profiles", "Profiles", "Upgrade List")
	--icon:Register("UpgradeListLDB", UpgradeListLDB, UpgradeListDBVar.profile.minimap)
	self:RegisterChatCommand("upgradelist", "ChatCommand")
	self:RegisterChatCommand("ul", "ChatCommand")
	--UpgradeListDBVar.RegisterCallback(self, "OnProfileChanged", "RefreshConfig")
	--UpgradeListDBVar.RegisterCallback(self, "OnProfileCopied", "RefreshConfig")
	--UpgradeListDBVar.RegisterCallback(self, "OnProfileReset", "RefreshConfig")
end
--[[
function UpgradeList:RefreshConfig()

	if UpgradeListDBVar.profile.minimap.hide then
		icon:Hide("UpgradeListLDB");
	else
		icon:Show("UpgradeListLDB");
	end
	
	if UpgradeListDBVar.profile.mainframe.lock then
		UpgradeListMainFrame:SetMovable(false);
		UpgradeListMainFrame:EnableMouse(false);
	else
		UpgradeListMainFrame:SetMovable(true);
		UpgradeListMainFrame:EnableMouse(true);
	end
	UpgradeListMainFrame:SetScale(UpgradeListDBVar.profile.mainframe.scale);
	if not DropDown_InitializeDropDown(self, level) then
		DropDown_InitializeDropDown(self, level)
	end
	
	
	
	UpgradeList:OnEnable()
end
--]]

function UpgradeList:ChatCommand(input)
	if not input or input:trim() == "" then
		InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
	else
		LibStub("AceConfigCmd-3.0").HandleCommand(UpgradeList, "ul", "upgradelist", input)
	end
end

function UpgradeList:MainFrameMovableGet(info)
	return UpgradeListDBVar.profile.mainframe.lock;
end

function UpgradeList:MainFrameMovableSet(info, value)
	UpgradeListDBVar.profile.mainframe.lock = not UpgradeListDBVar.profile.mainframe.lock;
	if UpgradeListDBVar.profile.mainframe.lock then
		UpgradeListMainFrame:SetMovable(false);
		UpgradeListMainFrame:EnableMouse(false);
	else
		UpgradeListMainFrame:SetMovable(true);
		UpgradeListMainFrame:EnableMouse(true);
	end
end

function UpgradeList:MinimapButtonGet(info)
	return UpgradeListDBVar.profile.minimap.hide;
end

function UpgradeList:MinimapButtonSet(info, value)
	UpgradeListDBVar.profile.minimap.hide = not UpgradeListDBVar.profile.minimap.hide;
	if UpgradeListDBVar.profile.minimap.hide then
		icon:Hide("UpgradeListLDB");
	else
		icon:Show("UpgradeListLDB");
	end
end

function UpgradeList:ScaleGet(info)
	return UpgradeListDBVar.profile.mainframe.scale;
end

function UpgradeList:ScaleSet(info, value)
	UpgradeListMainFrame:SetScale(value);
	UpgradeListDBVar.profile.mainframe.scale = value;
end

function UpgradeList:AlertMessagePotentialGet(info)
	return UpgradeListDBVar.profile.messages.potential;
end

function UpgradeList:AlertMessagePotentialSet(info, value)
	UpgradeListDBVar.profile.messages.potential = not UpgradeListDBVar.profile.messages.potential
end

function UpgradeList:AlertMessagePotentialDropdownGet(info)
	return UpgradeListDBVar.profile.messages.potentiallocation;
end

function UpgradeList:AlertMessagePotentialDropdownSet(info, value)
	UpgradeListDBVar.profile.messages.potentiallocation = value;
end

function UpgradeList:DisableMessagePotentialDropdown()
	if UpgradeListDBVar.profile.messages.potential then
		return false
	else
		return true
	end
end

function UpgradeList:AlertSoundPotentialGet(info)
	return UpgradeListDBVar.profile.sound.potential;
end

function UpgradeList:AlertSoundPotentialSet(info, value)
	UpgradeListDBVar.profile.sound.potential = not UpgradeListDBVar.profile.sound.potential
end

function UpgradeList:AlertSoundPotentialDropdownGet(info)
	return UpgradeListDBVar.profile.sound.potentialchoice;
end

function UpgradeList:AlertSoundPotentialDropdownSet(info, value)
	UpgradeListDBVar.profile.sound.potentialchoice = value;
end

function UpgradeList:DisableSoundPotentialDropdown()
	if UpgradeListDBVar.profile.sound.potential then
		return false
	else
		return true
	end
end

function UpgradeList:AlertMessageAcquiredGet(info)
	return UpgradeListDBVar.profile.messages.acquired;
end

function UpgradeList:AlertMessageAcquiredSet(info, value)
	UpgradeListDBVar.profile.messages.acquired = not UpgradeListDBVar.profile.messages.acquired
end

function UpgradeList:AlertMessageAcquiredDropdownGet(info)
	return UpgradeListDBVar.profile.messages.acquiredlocation;
end

function UpgradeList:AlertMessageAcquiredDropdownSet(info, value)
	UpgradeListDBVar.profile.messages.acquiredlocation = value;
end

function UpgradeList:DisableMessageAcquiredDropdown()
	if UpgradeListDBVar.profile.messages.acquired then
		return false
	else
		return true
	end
end

function UpgradeList:AlertSoundAcquiredGet(info)
	return UpgradeListDBVar.profile.sound.acquired;
end

function UpgradeList:AlertSoundAcquiredSet(info, value)
	UpgradeListDBVar.profile.sound.acquired = not UpgradeListDBVar.profile.sound.acquired
end

function UpgradeList:AlertSoundAcquiredDropdownGet(info)
	return UpgradeListDBVar.profile.sound.acquiredchoice;
end

function UpgradeList:AlertSoundAcquiredDropdownSet(info, value)
	UpgradeListDBVar.profile.sound.acquiredchoice = value;
end

function UpgradeList:DisableSoundAcquiredDropdown()
	if UpgradeListDBVar.profile.sound.acquired then
		return false
	else
		return true
	end
end

function UpgradeList:AutoDeleteGet(info)
	return UpgradeListDBVar.profile.autodelete.enable;
end

function UpgradeList:AutoDeleteSet(info, value)
	UpgradeListDBVar.profile.autodelete.enable = not UpgradeListDBVar.profile.autodelete.enable
end

function UpgradeList:AutoDeleteMessageGet(info)
	return UpgradeListDBVar.profile.autodelete.messages;
end

function UpgradeList:AutoDeleteMessageSet(info, value)
	UpgradeListDBVar.profile.autodelete.messages = not UpgradeListDBVar.profile.autodelete.messages
end

function UpgradeList:DisableAutoDeleteMessage()
	if UpgradeListDBVar.profile.autodelete.enable then
		return false
	else
		return true
	end
end

function UpgradeList:AutoDeleteMessageDropdownGet(info)
	return UpgradeListDBVar.profile.autodelete.messagelocation;
end

function UpgradeList:AutoDeleteMessageDropdownSet(info, value)
	UpgradeListDBVar.profile.autodelete.messagelocation = value;
end

function UpgradeList:DisableAutoDeleteMessageDropdown()
	if UpgradeListDBVar.profile.autodelete.enable and UpgradeListDBVar.profile.autodelete.messages then
		return false
	else
		return true
	end
end

function UpgradeList:IgnoreRingsGet(info)
	return UpgradeListDBVar.profile.ignoreflags.rings;
end

function UpgradeList:IgnoreRingsSet(info, value)
	UpgradeListDBVar.profile.ignoreflags.rings = not UpgradeListDBVar.profile.ignoreflags.rings;
end

function UpgradeList:DisableIgnoreRings()
	if UpgradeListDBVar.profile.autodelete.enable then
		return false
	else
		return true
	end
end

function UpgradeList:IgnoreTrinketsGet(info)
	return UpgradeListDBVar.profile.ignoreflags.trinkets;
end

function UpgradeList:IgnoreTrinketsSet(info, value)
	UpgradeListDBVar.profile.ignoreflags.trinkets = not UpgradeListDBVar.profile.ignoreflags.trinkets;
end

function UpgradeList:DisableIgnoreTrinkets()
	if UpgradeListDBVar.profile.autodelete.enable then
		return false
	else
		return true
	end
end

function UpgradeList:AutoDeleteChoiceDropdownGet(info)
	return UpgradeListDBVar.profile.autodelete.choice;
end

function UpgradeList:AutoDeleteChoiceDropdownSet(info, value)
	UpgradeListDBVar.profile.autodelete.choice = value;
end

function UpgradeList:DisableAutoDeleteChoiceDropdown()
	if UpgradeListDBVar.profile.autodelete.enable then
		return false
	else
		return true
	end
end

function UpgradeList:OneHandWeaponChoiceDropdownGet(info)
	return UpgradeListDBVar.profile.onehandweapon.choice;
end

function UpgradeList:OneHandWeaponChoiceDropdownSet(info, value)
	UpgradeListDBVar.profile.onehandweapon.choice = value;
end

function UpgradeList:ResetDatabase()
	UpgradeListDBVar.char.PrimarySpecList = defaults.char.PrimarySpecList;
	UpgradeListDBVar.char.SecondarySpecList = defaults.char.SecondarySpecList;
end

function UpgradeList:LoadItemIntoCache(item)
	GetItemInfo(item:GetText())
end

function UpgradeList:RefreshWindow(name, difficulty, instanceflag)
	if UpgradeListDBVar.char.SavedOptions.PrimaryCheck:GetChecked() then --If the primary checkbox is checked
		if PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == 1 then
			UpgradeListDBVar.char.SavedOptions.tableUL:SetData(UpgradeListDBVar.char.PrimarySpecList.Head.Row, true);
		elseif PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == 2 then
			UpgradeListDBVar.char.SavedOptions.tableUL:SetData(UpgradeListDBVar.char.PrimarySpecList.Neck.Row, true);
		elseif PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == 3 then
			UpgradeListDBVar.char.SavedOptions.tableUL:SetData(UpgradeListDBVar.char.PrimarySpecList.Shoulder.Row, true);
		elseif PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == 4 then
			UpgradeListDBVar.char.SavedOptions.tableUL:SetData(UpgradeListDBVar.char.PrimarySpecList.Back.Row, true);
		elseif PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == 5 then
			UpgradeListDBVar.char.SavedOptions.tableUL:SetData(UpgradeListDBVar.char.PrimarySpecList.Chest.Row, true);
		elseif PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == 6 then
			UpgradeListDBVar.char.SavedOptions.tableUL:SetData(UpgradeListDBVar.char.PrimarySpecList.Wrist.Row, true);
		elseif PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == 7 then
			UpgradeListDBVar.char.SavedOptions.tableUL:SetData(UpgradeListDBVar.char.PrimarySpecList.Hand.Row, true);
		elseif PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == 8 then
			UpgradeListDBVar.char.SavedOptions.tableUL:SetData(UpgradeListDBVar.char.PrimarySpecList.Waist.Row, true);
		elseif PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == 9 then
			UpgradeListDBVar.char.SavedOptions.tableUL:SetData(UpgradeListDBVar.char.PrimarySpecList.Leg.Row, true);
		elseif PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == 10 then
			UpgradeListDBVar.char.SavedOptions.tableUL:SetData(UpgradeListDBVar.char.PrimarySpecList.Foot.Row, true);
		elseif PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == 11 then
			UpgradeListDBVar.char.SavedOptions.tableUL:SetData(UpgradeListDBVar.char.PrimarySpecList.Ring.Row, true);
		elseif PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == 12 then
			UpgradeListDBVar.char.SavedOptions.tableUL:SetData(UpgradeListDBVar.char.PrimarySpecList.Trinket.Row, true);
		elseif PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == 13 then
			UpgradeListDBVar.char.SavedOptions.tableUL:SetData(UpgradeListDBVar.char.PrimarySpecList.Main.Row, true);
		elseif PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == 14 then
			UpgradeListDBVar.char.SavedOptions.tableUL:SetData(UpgradeListDBVar.char.PrimarySpecList.Off.Row, true);
		elseif PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == 15 then
			UpgradeListDBVar.char.SavedOptions.tableUL:SetData(UpgradeListDBVar.char.PrimarySpecList.Ranged.Row, true);
		elseif PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == 16 then
			UpgradeListDBVar.char.SavedOptions.tableUL:SetData(UpgradeListDBVar.char.PrimarySpecList.Other.Row, true);
		end
	else --The secondary checkbox is checked
		if PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == 1 then
			UpgradeListDBVar.char.SavedOptions.tableUL:SetData(UpgradeListDBVar.char.SecondarySpecList.Head.Row, true);
		elseif PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == 2 then
			UpgradeListDBVar.char.SavedOptions.tableUL:SetData(UpgradeListDBVar.char.SecondarySpecList.Neck.Row, true);
		elseif PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == 3 then
			UpgradeListDBVar.char.SavedOptions.tableUL:SetData(UpgradeListDBVar.char.SecondarySpecList.Shoulder.Row, true);
		elseif PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == 4 then
			UpgradeListDBVar.char.SavedOptions.tableUL:SetData(UpgradeListDBVar.char.SecondarySpecList.Back.Row, true);
		elseif PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == 5 then
			UpgradeListDBVar.char.SavedOptions.tableUL:SetData(UpgradeListDBVar.char.SecondarySpecList.Chest.Row, true);
		elseif PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == 6 then
			UpgradeListDBVar.char.SavedOptions.tableUL:SetData(UpgradeListDBVar.char.SecondarySpecList.Wrist.Row, true);
		elseif PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == 7 then
			UpgradeListDBVar.char.SavedOptions.tableUL:SetData(UpgradeListDBVar.char.SecondarySpecList.Hand.Row, true);
		elseif PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == 8 then
			UpgradeListDBVar.char.SavedOptions.tableUL:SetData(UpgradeListDBVar.char.SecondarySpecList.Waist.Row, true);
		elseif PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == 9 then
			UpgradeListDBVar.char.SavedOptions.tableUL:SetData(UpgradeListDBVar.char.SecondarySpecList.Leg.Row, true);
		elseif PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == 10 then
			UpgradeListDBVar.char.SavedOptions.tableUL:SetData(UpgradeListDBVar.char.SecondarySpecList.Foot.Row, true);
		elseif PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == 11 then
			UpgradeListDBVar.char.SavedOptions.tableUL:SetData(UpgradeListDBVar.char.SecondarySpecList.Ring.Row, true);
		elseif PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == 12 then
			UpgradeListDBVar.char.SavedOptions.tableUL:SetData(UpgradeListDBVar.char.SecondarySpecList.Trinket.Row, true);
		elseif PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == 13 then
			UpgradeListDBVar.char.SavedOptions.tableUL:SetData(UpgradeListDBVar.char.SecondarySpecList.Main.Row, true);
		elseif PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == 14 then
			UpgradeListDBVar.char.SavedOptions.tableUL:SetData(UpgradeListDBVar.char.SecondarySpecList.Off.Row, true);
		elseif PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == 15 then
			UpgradeListDBVar.char.SavedOptions.tableUL:SetData(UpgradeListDBVar.char.SecondarySpecList.Ranged.Row, true);
		elseif PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == 16 then
			UpgradeListDBVar.char.SavedOptions.tableUL:SetData(UpgradeListDBVar.char.SecondarySpecList.Other.Row, true);
		end
	end
	
	
	PanelTemplates_TabResize(Tab_1, 0, 93, 94);
	PanelTemplates_TabResize(Tab_2, 0, 93, 94);
	PanelTemplates_TabResize(Tab_3, 0, 93, 94);
	PanelTemplates_TabResize(Tab_4, 0, 93, 94);
	PanelTemplates_TabResize(Tab_5, 0, 93, 94);
	PanelTemplates_TabResize(Tab_6, 0, 93, 94);
	PanelTemplates_TabResize(Tab_7, 0, 93, 94);
	PanelTemplates_TabResize(Tab_8, 0, 93, 94);
	PanelTemplates_TabResize(Tab_9, 0, 93, 94);
	PanelTemplates_TabResize(Tab_10, 0, 93, 94);
	PanelTemplates_TabResize(Tab_11, 0, 93, 94);
	PanelTemplates_TabResize(Tab_12, 0, 93, 94);
	PanelTemplates_TabResize(Tab_13, 0, 102, 90);
	PanelTemplates_TabResize(Tab_14, 0, 94, 94);
	PanelTemplates_TabResize(Tab_15, 0, 94, 94);
	PanelTemplates_TabResize(Tab_16, 0, 94, 94);
	
end

function UpgradeList:OpenUpgradeList(self)
	if localizedItemType[Cloth] == nil then
		for itemType,itemNumber in pairs(WeaponAndArmorSubTypes) do
			_, _, _, _, _, _, localizedItemType[itemType] = GetItemInfo(tonumber(itemNumber));
		end
	end
	UpgradeList:StatusBarUpdate("UpgradeList v " .. UPGRADELIST_VERSION)
	UpgradeListMainFrame:Show()
	if UpgradeListDBVar.profile.mainframe.showoptions == false then
		OptionsFrame:Hide()
	end
	UpgradeList:RefreshWindow()
end

function UpgradeList:EnterItemIntoDatabase(ItemEntryBox, PrimarySpecRadioButton, UpgradeListMainFrame, priorityfield, NoteEntryBox, itemLink, itemLevel, itemType, itemSubType, itemEquipLoc, itemID)
	debugUL("Function: EnterItemIntoDatabase", DEBUGMODE);
	local AddToSpec
	local AddToSpecLong
	local priorityfieldnum = tonumber(priorityfield:GetText());
	if priorityfieldnum ~= nil then
		priorityfieldnum = math.floor(priorityfieldnum);
	end
	if PrimarySpecRadioButton:GetChecked() then
		AddToSpec = "Primary"
		AddToSpecLong = "PrimarySpecList"
	else
		AddToSpec = "Secondary"
		AddToSpecLong = "SecondarySpecList"
	end
	if priorityfieldnum == nil and priorityfield:GetText() ~= "" then
		UpgradeList:StatusBarUpdate("Priority field must be a number.", "red");
		priorityfield:HighlightText();
		return
	elseif UpgradeListDBVar.char.SavedOptions.SelectedInstance == nil then
		UpgradeList:StatusBarUpdate("You must select an item location.", "red");
		ItemEntryBox:ClearFocus();
		NoteEntryBox:ClearFocus();
		priorityfield:ClearFocus();
		return
	end
	if itemEquipLoc ~= "" and itemEquipLoc ~= "INVTYPE_BAG" and itemEquipLoc ~= "INVTYPE_TABARD" and itemEquipLoc ~= "INVTYPE_BODY" then
		for i = 1, #EquipLocationNumbers, 1 do --each tab, not including "Other" tab, it & 1H weapons have to be handled a little differently
			if itemEquipLoc == EquipLocationNumbers[i][1] then --Equip Location matched
				if priorityfield:GetText() == "" or priorityfieldnum > UpgradeListDBVar.char[AddToSpecLong][EquipLocationNumbers[i][3]].Count then --Priority # entered was blank or bigger than the list of items
					UpgradeListDBVar.char[AddToSpecLong][EquipLocationNumbers[i][3]].Row[UpgradeListDBVar.char[AddToSpecLong][EquipLocationNumbers[i][3]].Count] = { UpgradeListDBVar.char[AddToSpecLong][EquipLocationNumbers[i][3]].Count, itemLevel, itemLink, UpgradeListDBVar.char.SavedOptions.SelectedDifficulty, UpgradeListDBVar.char.SavedOptions.SelectedInstance, UpgradeListDBVar.char.SavedOptions.SelectedBoss, NoteEntryBox:GetText(), itemID };
				elseif priorityfieldnum <= UpgradeListDBVar.char[AddToSpecLong][EquipLocationNumbers[i][3]].Count and priorityfieldnum > 0 then --If the priority number is equal to or less than the list length and not zero
					table.insert(UpgradeListDBVar.char[AddToSpecLong][EquipLocationNumbers[i][3]].Row, priorityfieldnum, { priorityfieldnum, itemLevel, itemLink, UpgradeListDBVar.char.SavedOptions.SelectedDifficulty, UpgradeListDBVar.char.SavedOptions.SelectedInstance, UpgradeListDBVar.char.SavedOptions.SelectedBoss, NoteEntryBox:GetText(), itemID });
					for j = UpgradeListDBVar.char[AddToSpecLong][EquipLocationNumbers[i][3]].Count, priorityfieldnum, -1 do
						UpgradeListDBVar.char[AddToSpecLong][EquipLocationNumbers[i][3]].Row[j][1] = j;
					end
				end
				UpgradeListDBVar.char[AddToSpecLong][EquipLocationNumbers[i][3]].Count = UpgradeListDBVar.char[AddToSpecLong][EquipLocationNumbers[i][3]].Count + 1;
				UpgradeList:StatusBarUpdate("Successfully added " .. itemLink .. " to your " .. AddToSpec .. " Spec's " .. EquipLocationNumbers[i][3] .. " tab.", "green");
				if (PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == EquipLocationNumbers[i][2]) then --If you're viewing the tab you're adding to
					UpgradeList:RefreshWindow()
				end
				UpgradeListDBVar.char.SavedOptions.SelectedInstance = nil;
				UpgradeListDBVar.char.SavedOptions.SelectedBoss = nil;
				UpgradeListDBVar.char.SavedOptions.SelectedDifficulty = nil;
				ItemEntryBox:SetText("");
				ItemEntryBox:ClearFocus();
				NoteEntryBox:SetText("");
				NoteEntryBox:ClearFocus();
				priorityfield:SetText("");
				priorityfield:ClearFocus();
				UIDropDownMenu_SetText(DropDown_Button, "Location:");
			end
		end
		if itemEquipLoc == "INVTYPE_WEAPON" then
			WeaponInput:Show(); --After checking that priority field is a number and an item location is selected, show the WeaponInput frame.
			UpgradeList:DisableInput()
		end
	else --Adding to Other tab.
		if priorityfield:GetText() == "" or priorityfieldnum > UpgradeListDBVar.char[AddToSpecLong].Other.Count then --Priority # entered was blank or bigger than the list of items
			UpgradeListDBVar.char[AddToSpecLong].Other.Row[UpgradeListDBVar.char[AddToSpecLong].Other.Count] = { UpgradeListDBVar.char[AddToSpecLong].Other.Count, itemLevel, itemLink, UpgradeListDBVar.char.SavedOptions.SelectedDifficulty, UpgradeListDBVar.char.SavedOptions.SelectedInstance, UpgradeListDBVar.char.SavedOptions.SelectedBoss, NoteEntryBox:GetText(), itemID };
		elseif priorityfieldnum <= UpgradeListDBVar.char[AddToSpecLong].Other.Count and priorityfieldnum > 0 then --If the priority number is equal to or less than the list length and not zero
			table.insert(UpgradeListDBVar.char[AddToSpecLong].Other.Row, priorityfieldnum, { priorityfieldnum, itemLevel, itemLink, UpgradeListDBVar.char.SavedOptions.SelectedDifficulty, UpgradeListDBVar.char.SavedOptions.SelectedInstance, UpgradeListDBVar.char.SavedOptions.SelectedBoss, NoteEntryBox:GetText(), itemID });
			for i = UpgradeListDBVar.char[AddToSpecLong].Other.Count, priorityfieldnum, -1 do
				UpgradeListDBVar.char[AddToSpecLong].Other.Row[i][1] = i;
			end
		end
		UpgradeListDBVar.char[AddToSpecLong].Other.Count = UpgradeListDBVar.char[AddToSpecLong].Other.Count + 1;
		UpgradeList:StatusBarUpdate("Successfully added " .. itemLink .. " to your " .. AddToSpec .. " Spec's Other tab.", "green");
		if (PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == 16) then --If you're viewing the tab you're adding to
			UpgradeList:RefreshWindow()
		end
		UpgradeListDBVar.char.SavedOptions.SelectedInstance = nil;
		UpgradeListDBVar.char.SavedOptions.SelectedBoss = nil;
		UpgradeListDBVar.char.SavedOptions.SelectedDifficulty = nil;
		ItemEntryBox:SetText("");
		ItemEntryBox:ClearFocus();
		NoteEntryBox:SetText("");
		NoteEntryBox:ClearFocus();
		priorityfield:SetText("");
		priorityfield:ClearFocus();
		UIDropDownMenu_SetText(DropDown_Button, "Location:");
	end
end

function UpgradeList:ItemCanBeWorn(itemSubType, class)
	local equippableFlag = false
	for englishClass,equippable in pairs(EquippableItems) do
		if class == englishClass then
			for i = 1, #equippable, 1 do
				if itemSubType == localizedItemType[equippable[i]] then
					equippableFlag = true
				end
			end
		end
	end
	return equippableFlag
end

function UpgradeList:AddItem(ItemEntryBox, PrimarySpecRadioButton, UpgradeListMainFrame, priorityfield, NoteEntryBox)
	local _, itemLink, _, itemLevel, _, itemType, itemSubType, _, itemEquipLoc = GetItemInfo(ItemEntryBox:GetText());
	local ID, Color, Ltype;
	if itemEquipLoc ~= nil then
		_, _, Color, Ltype, ID = string.find(itemLink, "|?c?f?f?(%x*)|?H?([^:]*):?(%d+):?(%d*):?(%d*):?(%d*):?(%d*):?(%d*):?(%-?%d*):?(%-?%d*):?(%d*):?(%d*)|?h?%[?([^%[%]]*)%]?|?h?|?r?");
	end
	if itemEquipLoc == "" or itemEquipLoc == "INVTYPE_BAG" or itemEquipLoc == "INVTYPE_TABARD" or itemEqupLoc == "BODY" then --Non-equipable items
		UpgradeList:EnterItemIntoDatabase(ItemEntryBox, PrimarySpecRadioButton, UpgradeListMainFrame, priorityfield, NoteEntryBox, itemLink, itemLevel, itemType, itemSubType, itemEquipLoc, ID)
	elseif itemEquipLoc == nil then
		UpgradeList:StatusBarUpdate("Invalid Item", "red");
		ItemEntryBox:HighlightText();
		ItemEntryBox:SetFocus();
	else
		local _, englishClass = UnitClass("player");	
		if UpgradeList:ItemCanBeWorn(itemSubType, englishClass) then
			UpgradeList:EnterItemIntoDatabase(ItemEntryBox, PrimarySpecRadioButton, UpgradeListMainFrame, priorityfield, NoteEntryBox, itemLink, itemLevel, itemType, itemSubType, itemEquipLoc, ID)
		else
			UpgradeList:StatusBarUpdate("Your character cannot equip " .. itemLink, "red");
			ItemEntryBox:HighlightText();
			ItemEntryBox:SetFocus()
		end
	end
end

function UpgradeList:DeleteItem(deletefield, UpgradeListMainFrame)
	local DeleteFromSpec
	local DeleteFromSpecLong
	local tempItemLink
	local deletefieldnum = tonumber(deletefield:GetText());
	if deletefieldnum ~= nil then
		deletefieldnum = math.floor(deletefieldnum);
	end
	if UpgradeListDBVar.char.SavedOptions.PrimaryCheck:GetChecked() then
		DeleteFromSpec = "Primary"
		DeleteFromSpecLong = "PrimarySpecList"
	else
		DeleteFromSpec = "Secondary"
		DeleteFromSpecLong = "SecondarySpecList"
	end
	if deletefieldnum == nil and deletefield:GetText() ~= "" then
		UpgradeList:StatusBarUpdate("Delete field must be a number.", "red");
		deletefield:HighlightText();
		deletefield:SetFocus();
	elseif deletefieldnum == nil then
		UpgradeList:StatusBarUpdate("Nothing entered in delete field.", "red");
		deletefield:HighlightText();
		deletefield:SetFocus();
	else
		for i = 1, #TabNames, 1 do
			if PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == i then
				if deletefieldnum >= UpgradeListDBVar.char[DeleteFromSpecLong][TabNames[i][2]].Count or deletefieldnum == 0 then --Entered delete number isn't in the list
					UpgradeList:StatusBarUpdate("Delete field does not match an item in your " .. DeleteFromSpec .. " Spec's " .. TabNames[i][3] .. " list.", "red");
					deletefield:HighlightText();
					deletefield:SetFocus();
				elseif deletefieldnum ~= UpgradeListDBVar.char[DeleteFromSpecLong][TabNames[i][2]].Count - 1 and deletefieldnum > 0 then --Entered delete number is NOT the last one in the list
					tempItemLink = UpgradeListDBVar.char[DeleteFromSpecLong][TabNames[i][2]].Row[deletefieldnum][3]
					table.remove(UpgradeListDBVar.char[DeleteFromSpecLong][TabNames[i][2]].Row, deletefieldnum);
					UpgradeListDBVar.char[DeleteFromSpecLong][TabNames[i][2]].Count = UpgradeListDBVar.char[DeleteFromSpecLong][TabNames[i][2]].Count - 1;
					UpgradeList:StatusBarUpdate("Successfully deleted " .. tempItemLink .. " from your " .. DeleteFromSpec .. " Spec's " .. TabNames[i][3] .. " list.", "green");
					for j = UpgradeListDBVar.char[DeleteFromSpecLong][TabNames[i][2]].Row[deletefieldnum][1], UpgradeListDBVar.char[DeleteFromSpecLong][TabNames[i][2]].Count, 1 do
						UpgradeListDBVar.char[DeleteFromSpecLong][TabNames[i][2]].Row[j - 1][1] = j - 1;
					end
					deletefield:SetText("");
					deletefield:ClearFocus();
					UpgradeList:RefreshWindow()
				elseif deletefieldnum > 0 then --Deleting the last item in the list
					tempItemLink = UpgradeListDBVar.char[DeleteFromSpecLong][TabNames[i][2]].Row[deletefieldnum][3]
					table.remove(UpgradeListDBVar.char[DeleteFromSpecLong][TabNames[i][2]].Row, deletefieldnum);
					UpgradeListDBVar.char[DeleteFromSpecLong][TabNames[i][2]].Count = UpgradeListDBVar.char[DeleteFromSpecLong][TabNames[i][2]].Count - 1
					UpgradeList:StatusBarUpdate("Successfully deleted " .. tempItemLink .. " from your " .. DeleteFromSpec .. " Spec's " .. TabNames[i][3] .. " list.", "green");
					deletefield:SetText("");
					deletefield:ClearFocus();
					UpgradeList:RefreshWindow()
				else --Entered delete number is invalid
					UpgradeList:StatusBarUpdate(deletefieldnum .. " is not a valid number for deletion.", "red");
					deletefield:HighlightText()
					deletefield:SetFocus()
				end
			end
		end
	end
end

function UpgradeList:MoveItem(fromfield, tofield, UpgradeListMainFrame)
	debugUL("UpgradeList:MoveItem()", DEBUGMODE);
	local MoveSpec
	local MoveSpecLong
	local tempItemLink
	local fromfieldnum = tonumber(fromfield:GetText());
	local tofieldnum = tonumber(tofield:GetText());
	if fromfieldnum ~= nil and tofieldnum ~= nil then
		fromfieldnum = math.floor(fromfieldnum);
		tofieldnum = math.floor(tofieldnum);
	end
	if UpgradeListDBVar.char.SavedOptions.PrimaryCheck:GetChecked() then
		MoveSpec = "Primary"
		MoveSpecLong = "PrimarySpecList"
	else
		MoveSpec = "Secondary"
		MoveSpecLong = "SecondarySpecList"
	end
	if fromfieldnum == nil and fromfield:GetText() ~= "" then
		UpgradeList:StatusBarUpdate("From field must be a number.", "red");
		fromfield:HighlightText();
		fromfield:SetFocus();
	elseif tofieldnum == nil and tofield:GetText() ~= "" then
		UpgradeList:StatusBarUpdate("To field must be a number.", "red");
		tofield:HighlightText();
		tofield:SetFocus();
	elseif fromfieldnum == nil then
		UpgradeList:StatusBarUpdate("Nothing entered in From field.", "red");
		fromfield:HighlightText();
		fromfield:SetFocus();
	elseif tofieldnum == nil then
		UpgradeList:StatusBarUpdate("Nothing entered in To field.", "red");
		tofield:HighlightText();
		tofield:SetFocus();
	else
		for i = 1, #TabNames, 1 do
			if PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == i then
				if fromfieldnum >= UpgradeListDBVar.char[MoveSpecLong][TabNames[i][2]].Count or fromfieldnum == 0 then
					UpgradeList:StatusBarUpdate("From field does not match an item in your " .. MoveSpec .. " Spec's " .. TabNames[i][3] .. " list.", "red");
					fromfield:HighlightText();
					fromfield:SetFocus();
				elseif tofieldnum >= UpgradeListDBVar.char[MoveSpecLong][TabNames[i][2]].Count or tofieldnum == 0 then
					UpgradeList:StatusBarUpdate("To field does not match an item in your " .. MoveSpec .. " Spec's " .. TabNames[i][3] .. " list.", "red");
					tofield:HighlightText();
					tofield:SetFocus();
				elseif tofieldnum == fromfieldnum then
					UpgradeList:StatusBarUpdate("From field and To field match.  Cannot move any items.", "red");
					tofield:HighlightText();
					fromfield:HighlightText();
					tofield:SetFocus()
				else
					tempItemLink = UpgradeListDBVar.char[MoveSpecLong][TabNames[i][2]].Row[fromfieldnum][3]
					if fromfieldnum > tofieldnum then
						table.insert(UpgradeListDBVar.char[MoveSpecLong][TabNames[i][2]].Row, tofieldnum, UpgradeListDBVar.char[MoveSpecLong][TabNames[i][2]].Row[fromfieldnum])
						table.remove(UpgradeListDBVar.char[MoveSpecLong][TabNames[i][2]].Row, fromfieldnum + 1)
						for j = 1, #UpgradeListDBVar.char[MoveSpecLong][TabNames[i][2]].Row, 1 do
							UpgradeListDBVar.char[MoveSpecLong][TabNames[i][2]].Row[j][1] = j
						end
					else
						table.insert(UpgradeListDBVar.char[MoveSpecLong][TabNames[i][2]].Row, tofieldnum + 1, UpgradeListDBVar.char[MoveSpecLong][TabNames[i][2]].Row[fromfieldnum])
						table.remove(UpgradeListDBVar.char[MoveSpecLong][TabNames[i][2]].Row, fromfieldnum)
						for j = 1, #UpgradeListDBVar.char[MoveSpecLong][TabNames[i][2]].Row, 1 do
							UpgradeListDBVar.char[MoveSpecLong][TabNames[i][2]].Row[j][1] = j
						end
					end
					UpgradeList:StatusBarUpdate("Successfully moved " .. tempItemLink .. " to position " .. tofieldnum .. " in your " .. MoveSpec .. " Spec's " .. TabNames[i][3] .. " list.", "green");
					fromfield:SetText("");
					fromfield:ClearFocus();
					tofield:SetText("");
					tofield:ClearFocus();
					UpgradeList:RefreshWindow()
				end
			end
		end
	end
end

function UpgradeList:DeleteSectionOfItems(fromfield, tofield, UpgradeListMainFrame)
	debugUL("Function UpgradeList:DeleteSectionOfItems()", DEBUGMODE);
	local DeleteSectionSpec
	local DeleteSectionSpecLong
	local fromfieldnum = tonumber(fromfield:GetText());
	local tofieldnum = tonumber(tofield:GetText());
	if fromfieldnum ~= nil and tofieldnum ~= nil then
		fromfieldnum = math.floor(fromfieldnum);
		tofieldnum = math.floor(tofieldnum);
	end
	if UpgradeListDBVar.char.SavedOptions.PrimaryCheck:GetChecked() then
		DeleteSectionSpec = "Primary"
		DeleteSectionSpecLong = "PrimarySpecList"
	else
		DeleteSectionSpec = "Secondary"
		DeleteSectionSpecLong = "SecondarySpecList"
	end
	if fromfieldnum == nil and fromfield:GetText() ~= "" then
		UpgradeList:StatusBarUpdate("From field must be a number.", "red");
		multifromfield:HighlightText();
		multifromfield:SetFocus();
	elseif tofieldnum == nil and tofield:GetText() ~= "" then
		UpgradeList:StatusBarUpdate("To field must be a number.", "red");
		multitofield:HighlightText();
		multitofield:SetFocus();
	elseif fromfieldnum == nil then
		UpgradeList:StatusBarUpdate("Nothing entered in From field.", "red");
		multifromfield:HighlightText();
		multifromfield:SetFocus();
	elseif tofieldnum == nil then
		UpgradeList:StatusBarUpdate("Nothing entered in To field.", "red");
		multitofield:HighlightText();
		multitofield:SetFocus();
	else
		for i = 1, #TabNames, 1 do
			if PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == i then
				if fromfieldnum <= 0 or fromfieldnum >= UpgradeListDBVar.char[DeleteSectionSpecLong][TabNames[i][2]].Count then
					UpgradeList:StatusBarUpdate("From field does not match an item in your " .. DeleteSectionSpec .. " Spec's " .. TabNames[i][3] .. " list.", "red");
					multifromfield:HighlightText();
					multifromfield:SetFocus();
				elseif tofieldnum <= 0 or tofieldnum >= UpgradeListDBVar.char[DeleteSectionSpecLong][TabNames[i][2]].Count then
					UpgradeList:StatusBarUpdate("To field does not match an item in your " .. DeleteSectionSpec .. " Spec's " .. TabNames[i][3] .. " list.", "red");
					multitofield:HighlightText();
					multitofield:SetFocus();
				elseif tofieldnum == fromfieldnum then
					UpgradeList:StatusBarUpdate("You must use the Delete Item button and field above to delete only 1 item.", "red");
					deletefield:SetFocus();
				else
					if fromfieldnum > tofieldnum then
						local temp1 = tofieldnum
						tofieldnum = fromfieldnum
						fromfieldnum = temp1
					end
					for j = 1, (tofieldnum - fromfieldnum) + 1, 1 do
						table.remove(UpgradeListDBVar.char[DeleteSectionSpecLong][TabNames[i][2]].Row, fromfieldnum)
						UpgradeListDBVar.char[DeleteSectionSpecLong][TabNames[i][2]].Count = UpgradeListDBVar.char[DeleteSectionSpecLong][TabNames[i][2]].Count - 1;
						for k = 1, #UpgradeListDBVar.char[DeleteSectionSpecLong][TabNames[i][2]].Row, 1 do
							UpgradeListDBVar.char[DeleteSectionSpecLong][TabNames[i][2]].Row[k][1] = k
						end
					end
					UpgradeList:StatusBarUpdate("Successfully deleted all item from position " .. fromfieldnum .. " through position " .. tofieldnum .. " in your " .. DeleteSectionSpec .. " Spec's " .. TabNames[i][3] .. " list.", "green");
					multifromfield:SetText("");
					multifromfield:ClearFocus();
					multitofield:SetText("");
					multitofield:ClearFocus();
					UpgradeList:RefreshWindow()
				end
			end
		end
	end
end

function UpgradeList:RemoveMultipleItems(itemID, itemEquipLocation, spec, onehandweaponslot) --spec can be "mainspec" or "offspec" | onehandweaponslot can be "mainhand" or "offhand"
	debugUL("Function UpgradeList:RemoveMultipleItems()", DEBUGMODE);
	local RemoveMultipleItemsSpec
	local RemoveMultipleItemsSpecLong
	if spec == "mainspec" then
		RemoveMultipleItemsSpec = "Primary"
		RemoveMultipleItemsSpecLong = "PrimarySpecList"
	else
		RemoveMultipleItemsSpec = "Secondary"
		RemoveMultipleItemsSpecLong = "SecondarySpecList"
	end
	for i = 1, #EquipLocationNumbers, 1 do
		if itemEquipLocation == EquipLocationNumbers[i][1] then
			local rownum = 0;
			repeat
				rownum = rownum + 1;
			until (UpgradeListDBVar.char[RemoveMultipleItemsSpecLong][EquipLocationNumbers[i][3]].Row[rownum][8] == itemID or UpgradeListDBVar.char[RemoveMultipleItemsSpecLong][EquipLocationNumbers[i][3]].Row[rownum] == nil);
			if UpgradeListDBVar.profile.autodelete.messages then
				if UpgradeListDBVar.profile.autodelete.messagelocation == "chatwindow" then
					UpgradeList:ChatFrameMessage("Auto-deleting " .. UpgradeListDBVar.char[RemoveMultipleItemsSpecLong][EquipLocationNumbers[i][3]].Row[rownum][3] .. " from your " .. RemoveMultipleItemsSpec .. " Spec's list.");
				elseif UpgradeListDBVar.profile.autodelete.messagelocation == "scrollwindow" then
					UpgradeList:ScrollingWindowMessage("Auto-deleting " .. UpgradeListDBVar.char[RemoveMultipleItemsSpecLong][EquipLocationNumbers[i][3]].Row[rownum][3] .. " from your " .. RemoveMultipleItemsSpec .. " Spec's list.");
				end
			end
			for j = #UpgradeListDBVar.char[RemoveMultipleItemsSpecLong][EquipLocationNumbers[i][3]].Row, rownum, -1 do
				table.remove(UpgradeListDBVar.char[RemoveMultipleItemsSpecLong][EquipLocationNumbers[i][3]].Row);
				UpgradeListDBVar.char[RemoveMultipleItemsSpecLong][EquipLocationNumbers[i][3]].Count = UpgradeListDBVar.char[RemoveMultipleItemsSpecLong][EquipLocationNumbers[i][3]].Count - 1;
			end
		end
	end
	if itemEquipLocation == "INVTYPE_WEAPON" then
		local rownum = 0;
		if onehandweaponslot == "mainhand" then
			repeat
				rownum = rownum + 1;
			until (UpgradeListDBVar.char[RemoveMultipleItemsSpecLong].Main.Row[rownum][8] == itemID or UpgradeListDBVar.char[RemoveMultipleItemsSpecLong].Main.Row[rownum] == nil);
			if UpgradeListDBVar.profile.autodelete.messages then
				if UpgradeListDBVar.profile.autodelete.messagelocation == "chatwindow" then
					UpgradeList:ChatFrameMessage("Auto-deleting " .. UpgradeListDBVar.char[RemoveMultipleItemsSpecLong].Main.Row[rownum][3] .. " from your " .. RemoveMultipleItemsSpec .. " Spec's list.");
				elseif UpgradeListDBVar.profile.autodelete.messagelocation == "scrollwindow" then
					UpgradeList:ScrollingWindowMessage("Auto-deleting " .. UpgradeListDBVar.char[RemoveMultipleItemsSpecLong].Main.Row[rownum][3] .. " from your " .. RemoveMultipleItemsSpec .. " Spec's list.");
				end
			end
			for i = #UpgradeListDBVar.char[RemoveMultipleItemsSpecLong].Main.Row, rownum, -1 do
				table.remove(UpgradeListDBVar.char[RemoveMultipleItemsSpecLong].Main.Row);
				UpgradeListDBVar.char[RemoveMultipleItemsSpecLong].Main.Count = UpgradeListDBVar.char[RemoveMultipleItemsSpecLong].Main.Count - 1;
			end
		else
			repeat
				rownum = rownum + 1;
			until (UpgradeListDBVar.char[RemoveMultipleItemsSpecLong].Off.Row[rownum][8] == itemID or UpgradeListDBVar.char[RemoveMultipleItemsSpecLong].Off.Row[rownum] == nil);
			if UpgradeListDBVar.profile.autodelete.messages then
				if UpgradeListDBVar.profile.autodelete.messagelocation == "chatwindow" then
					UpgradeList:ChatFrameMessage("Auto-deleting " .. UpgradeListDBVar.char[RemoveMultipleItemsSpecLong].Off.Row[rownum][3] .. " from your " .. RemoveMultipleItemsSpec .. " Spec's list.");
				elseif UpgradeListDBVar.profile.autodelete.messagelocation == "scrollwindow" then
					UpgradeList:ScrollingWindowMessage("Auto-deleting " .. UpgradeListDBVar.char[RemoveMultipleItemsSpecLong].Off.Row[rownum][3] .. " from your " .. RemoveMultipleItemsSpec .. " Spec's list.");
				end
			end
			for i = #UpgradeListDBVar.char[RemoveMultipleItemsSpecLong].Off.Row, rownum, -1 do
				table.remove(UpgradeListDBVar.char[RemoveMultipleItemsSpecLong].Off.Row);
				UpgradeListDBVar.char[RemoveMultipleItemsSpecLong].Off.Count = UpgradeListDBVar.char[RemoveMultipleItemsSpecLong].Off.Count - 1;
			end
		end
	end
end

function UpgradeList:DecideMainorOffHand(itemID, itemEquipLocation, spec)
	MainHandorOffHand:Show();
	MainHandorOffHandOK:SetScript("OnClick", function (self, button, down)
		if WeaponInMain:GetChecked() then
			UpgradeList:RemoveMultipleItems(itemID, itemEquipLocation, spec, "mainhand");
		elseif WeaponInOff:GetChecked() then
			UpgradeList:RemoveMultipleItems(itemID, itemEquipLocation, spec, "offhand");
		end
		MainHandorOffHand:Hide();
		WeaponInOff:SetChecked(false);
		WeaponInMain:SetChecked(true);
		UpgradeList:EnableInput();
	end);
end

function UpgradeList:DecideSpec(itemID, itemEquipLocation, weaponinmainprimary, weaponinoffprimary, weaponinmainsecondary, weaponinoffsecondary)
	SpecInput:Show();
	SpecInputOK:SetScript("OnClick", function (self, button, down)
		if MainSpecCheck:GetChecked() then
			if weaponinmainprimary == true and weaponinoffprimary == true then
				SpecInput:Hide();
				UpgradeList:DecideMainorOffHand(itemID, itemEqiupLocation, "mainspec")
			else
				UpgradeList:RemoveMultipleItems(itemID, itemEquipLocation, "mainspec", nil);
			end
		elseif OffSpecCheck:GetChecked() then
			if weaponinmainsecondary == true and weaponinoffsecondary == true then
				SpecInput:Hide();
				UpgradeList:DecideMainorOffHand(itemID, itemEquipLocation, "offspec")
			else
				UpgradeList:RemoveMultipleItems(itemID, itemEquipLocation, "offspec", nil);
			end
		end
		SpecInput:Hide();
		OffSpecCheck:SetChecked(false);
		MainSpecCheck:SetChecked(true);
		UpgradeList:EnableInput();
	end);
end

function UpgradeList:AutoDelete(itemID, itemEquipLocation, primaryspecflag, secondaryspecflag, weaponinmainprimary, weaponinoffprimary, weaponinmainsecondary, weaponinoffsecondary)
	debugUL("Function UpgradeList:AutoDelete()", DEBUGMODE);
	if primaryspecflag == true and secondaryspecflag == true then --if acquired item was in both specs
		if UpgradeListDBVar.profile.autodelete.choice == 1 then
			--Consider the item for the player's main spec
			if weaponinmainprimary == true and weaponinoffprimary == true then --if the acquired item is a 1-handed weapon in the primary spec's Main Hand list and Off Hand list
				if UpgradeListDBVar.profile.onehandweapon.choice == 1 then --Consider the weapon as in the Main Hand list
					UpgradeList:RemoveMultipleItems(itemID, itemEquipLocation, "mainspec", "mainhand");
				elseif UpgradeListDBVar.profile.onehandweapon.choice == 2 then --Consider the weapon as in the Off Hand list
					UpgradeList:RemoveMultipleItems(itemID, itemEquipLocation, "mainspec", "offhand");
				elseif UpgradeListDBVar.profile.onehandweapon.choice == 4 then --Ask the user which list the weapon is in
					UpgradeList:DecideMainorOffHand(itemID, itemEquipLocation, "mainspec");
				else
					return;
				end
			elseif weaponinmainprimary == true then
				UpgradeList:RemoveMultipleItems(itemID, itemEquipLocation, "mainspec", "mainhand");
			elseif weaponinoffprimary == true then
				UpgradeList:RemoveMultipleItems(itemID, itemEquipLocation, "mainspec", "offhand");
			else
				UpgradeList:RemoveMultipleItems(itemID, itemEquipLocation, "mainspec", nil); --Not a 1-handed weapon in main spec
			end
		elseif UpgradeListDBVar.profile.autodelete.choice == 2 then
			--Consider the item for the player's off spec
			if weaponinmainsecondary == true and weaponinoffsecondary == true then --if the acquired item is a 1-handed weapon in the secondary spec's Main Hand list and Off Hand list
				if UpgradeListDBVar.profile.onehandweapon.choice == 1 then --Consider the weapon as in the Main Hand list
					UpgradeList:RemoveMultipleItems(itemID, itemEquipLocation, "offspec", "mainhand");
				elseif UpgradeListDBVar.profile.onehandweapon.choice == 2 then --Consider the weapon as in the Off Hand list
					UpgradeList:RemoveMultipleItems(itemID, itemEquipLocation, "offspec", "offhand");
				elseif UpgradeListDBVar.profile.onehandweapon.choice == 4 then --Ask the user which list the weapon is in
					UpgradeList:DecideMainorOffHand(itemID, itemEquipLocation, "offspec");
				else --Do nothing
					return;
				end
			elseif weaponinmainsecondary == true then
				UpgradeList:RemoveMultipleItems(itemID, itemEquipLocation, "offspec", "mainhand");
			elseif weaponinoffsecondary == true then
				UpgradeList:RemoveMultipleItems(itemID, itemEquipLocation, "offspec", "offhand");
			else
				UpgradeList:RemoveMultipleItems(itemID, itemEquipLocation, "offspec", nil); --Not a 1-handed weapon in off spec
			end
		elseif UpgradeListDBVar.profile.autodelete.choice == 4 then
			UpgradeList:DecideSpec(itemID, itemEquipLocation, weaponinmainprimary, weaponinoffprimary, weaponinmainsecondary, weaponinoffsecondary); --Figures out spec & if the item is a 1-handed weapon
		else
			return;
		end
	elseif primaryspecflag == true then --if acquired item was in main spec list only
		--Delete the item and those below it from the main spec list
		if weaponinmainprimary == true and weaponinoffprimary == true then --if the acquired item is a 1-handed weapon in the primary spec's Main Hand list and Off Hand list
			if UpgradeListDBVar.profile.onehandweapon.choice == 1 then --Consider the weapon as in the Main Hand list
				UpgradeList:RemoveMultipleItems(itemID, itemEquipLocation, "mainspec", "mainhand");
			elseif UpgradeListDBVar.profile.onehandweapon.choice == 2 then --Consider the weapon as in the Off Hand list
				UpgradeList:RemoveMultipleItems(itemID, itemEquipLocation, "mainspec", "offhand");
			elseif UpgradeListDBVar.profile.onehandweapon.choice == 4 then --Ask the user which list the weapon is in
				UpgradeList:DecideMainorOffHand(itemID, itemEquipLocation, "mainspec");
			else
				return;
			end
		elseif weaponinmainprimary == true and weaponinoffprimary == false then --if the acquired item is a 1-handed weapon in the primary spec's Main Hand list only
			UpgradeList:RemoveMultipleItems(itemID, itemEquipLocation, "mainspec", "mainhand");
		elseif weaponinmainprimary == false and weaponinoffprimary == true then --if the acquired item is a 1-handed weapon in the primary spec's Off Hand list only
			UpgradeList:RemoveMultipleItems(itemID, itemEquipLocation, "mainspec", "offhand");
		else
			UpgradeList:RemoveMultipleItems(itemID, itemEquipLocation, "mainspec", nil); --Not a 1-handed weapon in main spec
		end
	else -- if acquired item was in off spec list only
		--Delete the item and those below it from the off spec list
		if weaponinmainsecondary == true and weaponinoffsecondary == true then --if the acquired item is a 1-handed weapon in the secondary spec's Main Hand list and Off Hand list
			if UpgradeListDBVar.profile.onehandweapon.choice == 1 then --Consider the weapon as in the Main Hand list
				UpgradeList:RemoveMultipleItems(itemID, itemEquipLocation, "offspec", "mainhand");
			elseif UpgradeListDBVar.profile.onehandweapon.choice == 2 then --Consider the weapon as in the Off Hand list
				UpgradeList:RemoveMultipleItems(itemID, itemEquipLocation, "offspec", "offhand");
			elseif UpgradeListDBVar.profile.onehandweapon.choice == 4 then --Ask the user which list the weapon is in
				UpgradeList:DecideMainorOffHand(itemID, itemEquipLocation, "offspec");
			else
				return;
			end
		elseif weaponinmainsecondary == true and weaponinoffsecondary == false then --if the acquired item is a 1-handed weapon in the secondary spec's Main Hand list only
			UpgradeList:RemoveMultipleItems(itemID, itemEquipLocation, "offspec", "mainhand");
		elseif weaponinmainsecondary == false and weaponinoffsecondary == true then --if the acquired item is a 1-handed weapon in the secondary spec's Off Hand list only
			UpgradeList:RemoveMultipleItems(itemID, itemEquipLocation, "offspec", "offhand");
		else
			UpgradeList:RemoveMultipleItems(itemID, itemEquipLocation, "offspec", nil); --Not a 1-handed weapon in off spec
		end
	end
end

function UpgradeList:Compare(itemID, itemEquipLocation, attainability)
	local primaryspecflag = false;
	local secondaryspecflag = false;
	local weaponinmainprimary = false;
	local weaponinoffprimary = false;
	local weaponinmainsecondary = false;
	local weaponinoffsecondary = false;
	local otherflag = false;
	
	debugUL("Function UpgradeList:Compare()", DEBUGMODE);
	if DEBUGMODE then print(itemID) end
	if DEBUGMODE then print(itemEquipLocation) end
	if DEBUGMODE then print(attainability) end
	
	for i = 1, #EquipLocationNumbers, 1 do --Primary Spec Equippable Items Except INVTYPE_WEAPON, INVTYPE_BAG, INVTYPE_TABARD, and INVTYPE_BODY
		if itemEquipLocation == EquipLocationNumbers[i][1] then
			if UpgradeListDBVar.char.PrimarySpecList[EquipLocationNumbers[i][3]].Count > 1 then
				local j = 0;
				repeat
					j = j + 1
					if itemID == UpgradeListDBVar.char.PrimarySpecList[EquipLocationNumbers[i][3]].Row[j][8] then
						if attainability == "potential" then
							if UpgradeListDBVar.profile.messages.potential then
								if UpgradeListDBVar.profile.messages.potentiallocation == "chatwindow" then
									UpgradeList:ChatFrameMessage("Upgrade Detected for Primary Spec: " .. UpgradeListDBVar.char.PrimarySpecList[EquipLocationNumbers[i][3]].Row[j][3]);
								elseif UpgradeListDBVar.profile.messages.potentiallocation == "scrollwindow" then
									UpgradeList:ScrollingWindowMessage("Upgrade Detected for Primary Spec: " .. UpgradeListDBVar.char.PrimarySpecList[EquipLocationNumbers[i][3]].Row[j][3]);
								end
							end
							if UpgradeListDBVar.profile.sound.potential then
								PlaySoundFile(LSM:Fetch("sound", UpgradeListDBVar.profile.sound.potentialchoice))
							end
						else
							if UpgradeListDBVar.profile.messages.acquired then
								if UpgradeListDBVar.profile.messages.acquiredlocation == "chatwindow" then
									UpgradeList:ChatFrameMessage("Upgrade Acquired for Primary Spec: " .. UpgradeListDBVar.char.PrimarySpecList[EquipLocationNumbers[i][3]].Row[j][3]);
								elseif UpgradeListDBVar.profile.messages.acquiredlocation == "scrollwindow" then
									UpgradeList:ScrollingWindowMessage("Upgrade Acquired for Primary Spec: " .. UpgradeListDBVar.char.PrimarySpecList[EquipLocationNumbers[i][3]].Row[j][3]);
								end
							end
							if UpgradeListDBVar.profile.sound.acquired then
								PlaySoundFile(LSM:Fetch("sound", UpgradeListDBVar.profile.sound.acquiredchoice))
							end
							primaryspecflag = true;
						end
					end
				until (itemID == UpgradeListDBVar.char.PrimarySpecList[EquipLocationNumbers[i][3]].Row[j][8] or j == #UpgradeListDBVar.char.PrimarySpecList[EquipLocationNumbers[i][3]].Row);
			end
		end
	end
	if itemEquipLocation == "INVTYPE_WEAPON" then --Primary Spec INVTYPE_WEAPON
		if UpgradeListDBVar.char.PrimarySpecList.Main.Count > 1 then
			local i = 0;
			repeat
				i = i + 1;
				if itemID == UpgradeListDBVar.char.PrimarySpecList.Main.Row[i][8] then
					if attainability == "potential" then
						if UpgradeListDBVar.profile.messages.potential then
							if UpgradeListDBVar.profile.messages.potentiallocation == "chatwindow" then
								UpgradeList:ChatFrameMessage("Upgrade Detected for Primary Spec: " .. UpgradeListDBVar.char.PrimarySpecList.Main.Row[i][3]);
							elseif UpgradeListDBVar.profile.messages.potentiallocation == "scrollwindow" then
								UpgradeList:ScrollingWindowMessage("Upgrade Detected for Primary Spec: " .. UpgradeListDBVar.char.PrimarySpecList.Main.Row[i][3]);
							end
						end
						if UpgradeListDBVar.profile.sound.potential then
							PlaySoundFile(LSM:Fetch("sound", UpgradeListDBVar.profile.sound.potentialchoice))
						end
					else
						if UpgradeListDBVar.profile.messages.acquired then
							if UpgradeListDBVar.profile.messages.acquiredlocation == "chatwindow" then
								UpgradeList:ChatFrameMessage("Upgrade Acquired for Primary Spec: " .. UpgradeListDBVar.char.PrimarySpecList.Main.Row[i][3]);
							elseif UpgradeListDBVar.profile.messages.acquiredlocation == "scrollwindow" then
								UpgradeList:ScrollingWindowMessage("Upgrade Acquired for Primary Spec: " .. UpgradeListDBVar.char.PrimarySpecList.Main.Row[i][3]);
							end
						end
						if UpgradeListDBVar.profile.sound.acquired then
							PlaySoundFile(LSM:Fetch("sound", UpgradeListDBVar.profile.sound.acquiredchoice))
						end
						primaryspecflag = true;
						weaponinmainprimary = true;
					end
				end
			until (itemID == UpgradeListDBVar.char.PrimarySpecList.Main.Row[i][8] or i == #UpgradeListDBVar.char.PrimarySpecList.Main.Row);
		end
		if UpgradeListDBVar.char.PrimarySpecList.Off.Count > 1 then
			local i = 0;
			repeat
				i = i + 1;
				if itemID == UpgradeListDBVar.char.PrimarySpecList.Off.Row[i][8] then
					if attainability == "potential" then
						if UpgradeListDBVar.profile.messages.potential and weaponinmainprimary == false then
							if UpgradeListDBVar.profile.messages.potentiallocation == "chatwindow" then
								UpgradeList:ChatFrameMessage("Upgrade Detected for Primary Spec: " .. UpgradeListDBVar.char.PrimarySpecList.Off.Row[i][3]);
							elseif UpgradeListDBVar.profile.messages.potentiallocation == "scrollwindow" then
								UpgradeList:ScrollingWindowMessage("Upgrade Detected for Primary Spec: " .. UpgradeListDBVar.char.PrimarySpecList.Off.Row[i][3]);
							end
						end
						if UpgradeListDBVar.profile.sound.potential and weaponinmainprimary == false then
							PlaySoundFile(LSM:Fetch("sound", UpgradeListDBVar.profile.sound.potentialchoice))
						end
					else
						if UpgradeListDBVar.profile.messages.acquired and weaponinmainprimary == false then
							if UpgradeListDBVar.profile.messages.acquiredlocation == "chatwindow" then
								UpgradeList:ChatFrameMessage("Upgrade Acquired for Primary Spec: " .. UpgradeListDBVar.char.PrimarySpecList.Off.Row[i][3]);
							elseif UpgradeListDBVar.profile.messages.acquiredlocation == "scrollwindow" then
								UpgradeList:ScrollingWindowMessage("Upgrade Acquired for Primary Spec: " .. UpgradeListDBVar.char.PrimarySpecList.Off.Row[i][3]);
							end
						end
						if UpgradeListDBVar.profile.sound.acquired and weaponinmainprimary == false then
							PlaySoundFile(LSM:Fetch("sound", UpgradeListDBVar.profile.sound.acquiredchoice))
						end
						primaryspecflag = true;
						weaponinoffprimary = true;
					end
				end
			until (itemID == UpgradeListDBVar.char.PrimarySpecList.Off.Row[i][8] or i == #UpgradeListDBVar.char.PrimarySpecList.Off.Row);
		end
	end
	if itemEquipLocation == "" or itemEquipLocation == "INVTYPE_BAG" or itemEquipLocation == "INVTYPE_TABARD" or itemEquipLocation == "INVTYPE_BODY" then --Primary Spec "Other" Tab
		if UpgradeListDBVar.char.PrimarySpecList.Other.Count > 1 then
			local i = 0;
			repeat
				i = i + 1;
				if itemID == UpgradeListDBVar.char.PrimarySpecList.Other.Row[i][8] then
					if attainability == "potential" then
						if UpgradeListDBVar.profile.messages.potential then
							if UpgradeListDBVar.profile.messages.potentiallocation == "chatwindow" then
								UpgradeList:ChatFrameMessage("Item Detected in Primary Spec's \"Other\" Tab: " .. UpgradeListDBVar.char.PrimarySpecList.Other.Row[i][3]);
							elseif UpgradeListDBVar.profile.messages.potentiallocation == "scrollwindow" then
								UpgradeList:ScrollingWindowMessage("Item Detected in Primary Spec's \"Other\" Tab: " .. UpgradeListDBVar.char.PrimarySpecList.Other.Row[i][3]);
							end
						end
						if UpgradeListDBVar.profile.sound.potential then
							PlaySoundFile(LSM:Fetch("sound", UpgradeListDBVar.profile.sound.potentialchoice))
						end
					else
						if UpgradeListDBVar.profile.messages.acquired then					
							if UpgradeListDBVar.profile.messages.acquiredlocation == "chatwindow" then
								UpgradeList:ChatFrameMessage("Upgrade Acquired for Primary Spec: " .. UpgradeListDBVar.char.PrimarySpecList.Other.Row[i][3]);
							elseif UpgradeListDBVar.profile.messages.acquiredlocation == "scrollwindow" then
								UpgradeList:ScrollingWindowMessage("Upgrade Acquired for Primary Spec: " .. UpgradeListDBVar.char.PrimarySpecList.Other.Row[i][3]);
							end
						end
						if UpgradeListDBVar.profile.messages.acquired then
							PlaySoundFile(LSM:Fetch("sound", UpgradeListDBVar.profile.sound.acquiredchoice))
						end
						otherflag = true;
					end
				end
			until (itemID == UpgradeListDBVar.char.PrimarySpecList.Other.Row[i][8] or i == #UpgradeListDBVar.char.PrimarySpecList.Other.Row);
		end
	end
	if GetNumTalentGroups("player") > 1 then --If the player has 2 talent specs, search through the secondary spec list
		for i = 1, #EquipLocationNumbers, 1 do --Secondary Spec Equippable Items Except INVTYPE_WEAPON, INVTYPE_BAG, INVTYPE_TABARD, and INVTYPE_BODY
			if itemEquipLocation == EquipLocationNumbers[i][1] then
				if UpgradeListDBVar.char.SecondarySpecList[EquipLocationNumbers[i][3]].Count > 1 then
					local j = 0;
					repeat
						j = j + 1
						if itemID == UpgradeListDBVar.char.SecondarySpecList[EquipLocationNumbers[i][3]].Row[j][8] then
							if attainability == "potential" then
								if UpgradeListDBVar.profile.messages.potential then
									if UpgradeListDBVar.profile.messages.potentiallocation == "chatwindow" then
										UpgradeList:ChatFrameMessage("Upgrade Detected for Secondary Spec: " .. UpgradeListDBVar.char.SecondarySpecList[EquipLocationNumbers[i][3]].Row[j][3]);
									elseif UpgradeListDBVar.profile.messages.potentiallocation == "scrollwindow" then
										UpgradeList:ScrollingWindowMessage("Upgrade Detected for Secondary Spec: " .. UpgradeListDBVar.char.SecondarySpecList[EquipLocationNumbers[i][3]].Row[j][3]);
									end
								end
								if UpgradeListDBVar.profile.sound.potential then
									PlaySoundFile(LSM:Fetch("sound", UpgradeListDBVar.profile.sound.potentialchoice))
								end
							else
								if UpgradeListDBVar.profile.messages.acquired then
									if UpgradeListDBVar.profile.messages.acquiredlocation == "chatwindow" then
										UpgradeList:ChatFrameMessage("Upgrade Acquired for Secondary Spec: " .. UpgradeListDBVar.char.SecondarySpecList[EquipLocationNumbers[i][3]].Row[j][3]);
									elseif UpgradeListDBVar.profile.messages.acquiredlocation == "scrollwindow" then
										UpgradeList:ScrollingWindowMessage("Upgrade Acquired for Secondary Spec: " .. UpgradeListDBVar.char.SecondarySpecList[EquipLocationNumbers[i][3]].Row[j][3]);
									end
								end
								if UpgradeListDBVar.profile.sound.acquired then
									PlaySoundFile(LSM:Fetch("sound", UpgradeListDBVar.profile.sound.acquiredchoice))
								end
								Secondaryspecflag = true;
							end
						end
					until (itemID == UpgradeListDBVar.char.SecondarySpecList[EquipLocationNumbers[i][3]].Row[j][8] or j == #UpgradeListDBVar.char.SecondarySpecList[EquipLocationNumbers[i][3]].Row);
				end
			end
		end
		if itemEquipLocation == "INVTYPE_WEAPON" then --Secondary Spec INVTYPE_WEAPON
			if UpgradeListDBVar.char.SecondarySpecList.Main.Count > 1 then
				local i = 0;
				repeat
					i = i + 1;
					if itemID == UpgradeListDBVar.char.SecondarySpecList.Main.Row[i][8] then
						if attainability == "potential" then
							if UpgradeListDBVar.profile.messages.potential then
								if UpgradeListDBVar.profile.messages.potentiallocation == "chatwindow" then
									UpgradeList:ChatFrameMessage("Upgrade Detected for Secondary Spec: " .. UpgradeListDBVar.char.SecondarySpecList.Main.Row[i][3]);
								elseif UpgradeListDBVar.profile.messages.potentiallocation == "scrollwindow" then
									UpgradeList:ScrollingWindowMessage("Upgrade Detected for Secondary Spec: " .. UpgradeListDBVar.char.SecondarySpecList.Main.Row[i][3]);
								end
							end
							if UpgradeListDBVar.profile.sound.potential and Secondaryspecflag == false then
								PlaySoundFile(LSM:Fetch("sound", UpgradeListDBVar.profile.sound.potentialchoice))
							end
						else
							if UpgradeListDBVar.profile.messages.acquired and Secondaryspecflag == false then
								if UpgradeListDBVar.profile.messages.acquiredlocation == "chatwindow" then
									UpgradeList:ChatFrameMessage("Upgrade Acquired for Secondary Spec: " .. UpgradeListDBVar.char.SecondarySpecList.Main.Row[i][3]);
								elseif UpgradeListDBVar.profile.messages.acquiredlocation == "scrollwindow" then
									UpgradeList:ScrollingWindowMessage("Upgrade Acquired for Secondary Spec: " .. UpgradeListDBVar.char.SecondarySpecList.Main.Row[i][3]);
								end
							end
							if UpgradeListDBVar.profile.sound.acquired and Secondaryspecflag == false then
								PlaySoundFile(LSM:Fetch("sound", UpgradeListDBVar.profile.sound.acquiredchoice))
							end
							secondaryspecflag = true;
							weaponinmainsecondary = true;
						end
					end
				until (itemID == UpgradeListDBVar.char.SecondarySpecList.Main.Row[i][8] or i == #UpgradeListDBVar.char.SecondarySpecList.Main.Row);
			end
			if UpgradeListDBVar.char.SecondarySpecList.Off.Count > 1 then
				local i = 0;
				repeat
					i = i + 1;
					if itemID == UpgradeListDBVar.char.SecondarySpecList.Off.Row[i][8] then
						if attainability == "potential" then
							if UpgradeListDBVar.profile.messages.potential and weaponinmainsecondary == false then
								if UpgradeListDBVar.profile.messages.potentiallocation == "chatwindow" then
									UpgradeList:ChatFrameMessage("Upgrade Detected for Secondary Spec: " .. UpgradeListDBVar.char.SecondarySpecList.Off.Row[i][3]);
								elseif UpgradeListDBVar.profile.messages.potentiallocation == "scrollwindow" then
									UpgradeList:ScrollingWindowMessage("Upgrade Detected for Secondary Spec: " .. UpgradeListDBVar.char.SecondarySpecList.Off.Row[i][3]);
								end
							end
							if UpgradeListDBVar.profile.sound.potential and Secondaryspecflag == false and weaponinmainsecondary == false then
								PlaySoundFile(LSM:Fetch("sound", UpgradeListDBVar.profile.sound.potentialchoice))
							end
						else
							if UpgradeListDBVar.profile.messages.acquired and Secondaryspecflag == false and weaponinmainsecondary == false then
								if UpgradeListDBVar.profile.messages.acquiredlocation == "chatwindow" then
									UpgradeList:ChatFrameMessage("Upgrade Acquired for Secondary Spec: " .. UpgradeListDBVar.char.SecondarySpecList.Off.Row[i][3]);
								elseif UpgradeListDBVar.profile.messages.acquiredlocation == "scrollwindow" then
									UpgradeList:ScrollingWindowMessage("Upgrade Acquired for Secondary Spec: " .. UpgradeListDBVar.char.SecondarySpecList.Off.Row[i][3]);
								end
							end
							if UpgradeListDBVar.profile.sound.acquired and Secondaryspecflag == false and weaponinmainsecondary == false then
								PlaySoundFile(LSM:Fetch("sound", UpgradeListDBVar.profile.sound.acquiredchoice))
							end
							secondaryspecflag = true;
							weaponinoffsecondary = true;
						end
					end
				until (itemID == UpgradeListDBVar.char.SecondarySpecList.Off.Row[i][8] or i == #UpgradeListDBVar.char.SecondarySpecList.Off.Row);
			end
		end
		if itemEquipLocation == "" or itemEquipLocation == "INVTYPE_BAG" or itemEquipLocation == "INVTYPE_TABARD" or itemEquipLocation == "INVTYPE_BODY" then --Secondary Spec "Other" Tab
			if UpgradeListDBVar.char.SecondarySpecList.Other.Count > 1 then
				local i = 0;
				repeat
					i = i + 1;
					if itemID == UpgradeListDBVar.char.SecondarySpecList.Other.Row[i][8] then
						if attainability == "potential" then
							if UpgradeListDBVar.profile.messages.potential and otherflag == false then
								if UpgradeListDBVar.profile.messages.potentiallocation == "chatwindow" then
									UpgradeList:ChatFrameMessage("Item Detected in Secondary Spec's \"Other\" Tab: " .. UpgradeListDBVar.char.SecondarySpecList.Other.Row[i][3]);
								elseif UpgradeListDBVar.profile.messages.potentiallocation == "scrollwindow" then
									UpgradeList:ScrollingWindowMessage("Item Detected in Secondary Spec's \"Other\" Tab: " .. UpgradeListDBVar.char.SecondarySpecList.Other.Row[i][3]);
								end
							end
							if UpgradeListDBVar.profile.sound.potential then
								PlaySoundFile(LSM:Fetch("sound", UpgradeListDBVar.profile.sound.potentialchoice))
							end
						else
							if UpgradeListDBVar.profile.messages.acquired then					
								if UpgradeListDBVar.profile.messages.acquiredlocation == "chatwindow" then
									UpgradeList:ChatFrameMessage("Item Acquired in Secondary Spec's \"Other\" Tab: " .. UpgradeListDBVar.char.SecondarySpecList.Other.Row[i][3]);
								elseif UpgradeListDBVar.profile.messages.acquiredlocation == "scrollwindow" then
									UpgradeList:ScrollingWindowMessage("Item Acquired in Secondary Spec's \"Other\" Tab: " .. UpgradeListDBVar.char.SecondarySpecList.Other.Row[i][3]);
								end
							end
							if UpgradeListDBVar.profile.messages.acquired then
								PlaySoundFile(LSM:Fetch("sound", UpgradeListDBVar.profile.sound.acquiredchoice))
							end
							otherflag = true;
						end
					end
				until (itemID == UpgradeListDBVar.char.SecondarySpecList.Other.Row[i][8] or i == #UpgradeListDBVar.char.SecondarySpecList.Other.Row);
			end
		end
	end
	if otherflag == false and (primaryspecflag == true or secondaryspecflag == true) then --if item was not in "Other" tab ("Other" tab does not auto-delete) and was acquired
		if UpgradeListDBVar.profile.autodelete.enable then
			if itemEquipLocation ~= "INVTYPE_FINGER" and itemEquipLocation ~= "INVTYPE_TRINKET" or (itemEquipLocation == "INVTYPE_FINGER" and UpgradeListDBVar.profile.ignoreflags.rings == false) or (itemEquipLocation == "INVTYPE_TRINKET" and UpgradeListDBVar.profile.ignoreflags.trinkets == false) then --if the acquired item is not a ring or trinket, or if it was a ring and the ignore rings checkbox is unchecked, or if it was a trinket and the ignore trinkets checkbox is unchecked
				UpgradeList:AutoDelete(itemID, itemEquipLocation, primaryspecflag, secondaryspecflag, weaponinmainprimary, weaponinoffprimary, weaponinmainsecondary, weaponinoffsecondary);
			elseif itemEquipLocation == "INVTYPE_FINGER" and UpgradeListDBVar.profile.ignoreflags.rings == true then
				local _, itemLink = GetItemInfo(itemID);
				UpgradeList:ChatFrameMessage("" .. itemLink .. " acquired, remember to manually update your Ring list!");
			elseif itemEquipLocation == "INVTYPE_TRINKET" and UpgradeListDBVar.profile.ignoreflags.trinkets == true then
				local _, itemLink = GetItemInfo(itemID);
				UpgradeList:ChatFrameMessage("" .. itemLink .. " acquired, remember to manually update your Trinket list!");
			end
		end
	elseif otherflag == true then
		local _, itemLink = GetItemInfo(itemID);
		UpgradeList:ChatFrameMessage("" .. itemLink .. " acquired, remember to manually update your Other list!");
	end
end

function UpgradeList:ToggleOptions()
	if OptionsFrame:IsShown() then
		OptionsFrame:Hide()
		UpgradeListDBVar.profile.mainframe.showoptions = false;
	else
		OptionsFrame:Show()
		UpgradeListDBVar.profile.mainframe.showoptions = true;
	end
end

function UpgradeList:CHAT_MSG_LOOT(eventName, arg1)
	if string.match(arg1, 'You receive loot: ') or string.match(arg1, 'You receive item: ') or string.match(arg1, 'You create: ') then
		debugUL("Chat message You receive loot or You receive item or You create detected: ", DEBUGMODE);
		local minitable = { strsplit(" [", arg1) };
		local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture, itemSellPrice;
		for i = 1, #minitable, 1 do
			if string.sub(minitable[i], 1, 1) == "|" then --In minitable, if the first character of a variable is a "|" symbol, that is the item string, so use that entry of minitable to get the itemEquipLoc
				itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture, itemSellPrice = GetItemInfo(minitable[i]);
				if DEBUGMODE then print(itemLink) end
			end
		end
		local _, _, Color, Ltype, ID = string.find(itemLink, "|?c?f?f?(%x*)|?H?([^:]*):?(%d+):?(%d*):?(%d*):?(%d*):?(%d*):?(%d*):?(%-?%d*):?(%-?%d*):?(%d*):?(%d*)|?h?%[?([^%[%]]*)%]?|?h?|?r?");
		UpgradeList:Compare(ID, itemEquipLoc, "acquired");
	end
end

function UpgradeList:CHAT_MSG_SYSTEM(eventName, arg1)
	if string.match(arg1, 'Received item:') or string.match(arg1, 'Received .. of item:') then
		debugUL("Chat message Received item or Received # of item detected: ", DEBUGMODE);
		UpgradeList:CHAT_MSG_LOOT(self, arg1);
		local minitable = { strsplit(" [", arg1) };
		local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture, itemSellPrice;
		for i = 1, #minitable, 1 do
			if string.sub(minitable[i], 1, 1) == "|" then --In minitable, if the first character of a variable is a "|" symbol, that is the item string, so use that entry of minitable to get the itemEquipLoc
				itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture, itemSellPrice = GetItemInfo(minitable[i]);
				if DEBUGMODE then print(itemLink) end
			end
		end
		local _, _, Color, Ltype, ID = string.find(itemLink, "|?c?f?f?(%x*)|?H?([^:]*):?(%d+):?(%d*):?(%d*):?(%d*):?(%d*):?(%d*):?(%-?%d*):?(%-?%d*):?(%d*):?(%d*)|?h?%[?([^%[%]]*)%]?|?h?|?r?");
		UpgradeList:Compare(ID, itemEquipLoc, "acquired");	
	end
end

function UpgradeList:LOOT_OPENED()
	debugUL("Loot window opened: ", DEBUGMODE);
	for index = 1, GetNumLootItems() do
		if (LootSlotIsItem(index)) then
			local itemLink = GetLootSlotLink(index);
			if DEBUGMODE then print(itemLink) end
			local _, _, Color, Ltype, ID = string.find(itemLink, "|?c?f?f?(%x*)|?H?([^:]*):?(%d+):?(%d*):?(%d*):?(%d*):?(%d*):?(%d*):?(%-?%d*):?(%-?%d*):?(%d*):?(%d*)|?h?%[?([^%[%]]*)%]?|?h?|?r?");
			local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture, itemSellPrice = GetItemInfo(itemLink);
			UpgradeList:Compare(ID, itemEquipLoc, "potential");
		end
	end
end

function UpgradeList:START_LOOT_ROLL(eventName, arg1, arg2) --This and Loot Opened fire when looting something in a group, figure out how to only have Loot Opened fire when opening loot.
	debugUL("Item being rolled on: ", DEBUGMODE);
	local itemLink = GetLootRollItemLink(arg1);
	if DEBUGMODE then print(itemLink) end
	local _, _, Color, Ltype, ID = string.find(itemLink, "|?c?f?f?(%x*)|?H?([^:]*):?(%d+):?(%d*):?(%d*):?(%d*):?(%d*):?(%d*):?(%-?%d*):?(%-?%d*):?(%d*):?(%d*)|?h?%[?([^%[%]]*)%]?|?h?|?r?");
	local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture, itemSellPrice = GetItemInfo(itemLink);
	UpgradeList:Compare(ID, itemEquipLoc, "potential");
end

function UpgradeList:PLAYER_ENTERING_WORLD()
	local inInstance, instanceType = IsInInstance();
	if inInstance == 1 and (instanceType == "party" or instanceType == "raid") then --if you're in an instance
		local instanceflag = true;
		local name, instancetype, difficultyIndex, difficultyName, maxPlayers, dynamicDifficulty, isDynamic = GetInstanceInfo();
		if instanceType == "party" and difficultyIndex == 1 then --Normal 5 mans
			UpgradeList:RefreshWindow(name, "Normal", instanceflag);
		elseif instanceType == "party" and difficultyIndex == 2 then --Heroic 5 mans
			UpgradeList:RefreshWindow(name, "Heoric", instanceflag);
		elseif instanceType == "raid" and (difficultyIndex == 1 or difficultyIndex == 2) then --10/25 man Normal raids
			UpgradeList:RefreshWindow(name, "Normal", instanceflag);
		elseif instanceType == "raid" and (difficultyIndex == 3 or difficultyIndex == 4) then --10/25 man Heroic raids
			UpgradeList:RefreshWindow(name, "Heroic", instanceflag);
		end
	end
end

local function ScanQuestItems()
	local num = GetNumQuestChoices();
	if num > 0 then
		for i = 1, num, 1 do
			local itemLink = GetQuestItemLink("choice", i);
			if itemLink == nil then
				print("UpgradeList: Error occurred while scanning quest items.");
			else
				debugUL("Item award detected from quest:", DEBUGMODE);
				if DEBUGMODE then print(itemLink) end
				local _, _, Color, Ltype, ID = string.find(itemLink, "|?c?f?f?(%x*)|?H?([^:]*):?(%d+):?(%d*):?(%d*):?(%d*):?(%d*):?(%d*):?(%-?%d*):?(%-?%d*):?(%d*):?(%d*)|?h?%[?([^%[%]]*)%]?|?h?|?r?");
				local _, _, _, _, _, _, _, _, itemEquipLoc, _, _ = GetItemInfo(itemLink);
				UpgradeList:Compare(ID, itemEquipLoc, "potential");
			end
		end
	end
end

function UpgradeList:QUEST_DETAIL()
	UpgradeList_wait(.5, ScanQuestItems)
end

function UpgradeList:QUEST_COMPLETE()
	UpgradeList_wait(.5, ScanQuestItems)
end

function UpgradeList:CHAT_MSG_RAID(arg1, arg2, arg3)
	local lootmethod, masterlooterPartyID, masterlooterRaidID = GetLootMethod()
	if lootmethod == "master" and UnitInRaid("player") then
		local name = GetRaidRosterInfo(masterlooterRaidID)
		if arg3 == name and name ~= UnitName("player") then
			debugUL("Master looter chat msg in raid:", DEBUGMODE);
			local _, itemLink, _, _, _, _, _, _, itemEquipLoc, _, _ = GetItemInfo(arg2);
			if itemLink then
				debugUL("Master looter has linked an item in raid chat:", DEBUGMODE);
				if DEBUGMODE then print(itemLink) end
				local _, _, Color, Ltype, ID = string.find(itemLink, "|?c?f?f?(%x*)|?H?([^:]*):?(%d+):?(%d*):?(%d*):?(%d*):?(%d*):?(%d*):?(%-?%d*):?(%-?%d*):?(%d*):?(%d*)|?h?%[?([^%[%]]*)%]?|?h?|?r?");
				UpgradeList:Compare(ID, itemEquipLoc, "potential");
			end
		end
	end
end

function UpgradeList:CHAT_MSG_RAID_LEADER(arg1, arg2, arg3)
	UpgradeList:CHAT_MSG_RAID(arg1, arg2, arg3)
end

function UpgradeList:CHAT_MSG_RAID_WARNING(arg1, arg2, arg3)
	local lootmethod, masterlooterPartyID, masterlooterRaidID = GetLootMethod()
	if lootmethod == "master" and UnitInRaid("player") then
		local name = GetRaidRosterInfo(masterlooterRaidID)
		if arg3 == name and name ~= UnitName("player") then
			debugUL("Master looter raid warning:", DEBUGMODE);
			local _, itemLink, _, _, _, _, _, _, itemEquipLoc, _, _ = GetItemInfo(arg2);
			if itemLink then
				debugUL("Master looter has linked an item in raid warning:", DEBUGMODE);
				if DEBUGMODE then print(itemLink) end
				local _, _, Color, Ltype, ID = string.find(itemLink, "|?c?f?f?(%x*)|?H?([^:]*):?(%d+):?(%d*):?(%d*):?(%d*):?(%d*):?(%d*):?(%-?%d*):?(%-?%d*):?(%d*):?(%d*)|?h?%[?([^%[%]]*)%]?|?h?|?r?");
				UpgradeList:Compare(ID, itemEquipLoc, "potential");
			end
		end
	end
end

function UpgradeList:ACHIEVEMENT_EARNED(eventName, arg1) --Enables the checkbox and radio button for the secondary spec if the player earns the Duel-Talent specialization achievement.
	if arg1 == 2716 then
		debugUL("Dual-spec achievement event registered", DEBUGMODE);
		UpgradeListDBVar.char.SavedOptions.SecondaryCheck:Enable();
		SecondarySpecRadioButton:Enable();
	end
end

function UpgradeList:ChatEdit_InsertLink(lnk) --Called when a player shift-clicks an item, if the ItemEntryBox has focus, it will add the item's link to the box.
	if ItemEntryBox:HasFocus() then
		ItemEntryBox:SetText(lnk)
	end
end


function UpgradeList:OnEnable()
	--Register for Events
	UpgradeList:RegisterEvent("CHAT_MSG_LOOT")
	UpgradeList:RegisterEvent("CHAT_MSG_SYSTEM")
	UpgradeList:RegisterEvent("LOOT_OPENED")
	UpgradeList:RegisterEvent("START_LOOT_ROLL")
	UpgradeList:RegisterEvent("PLAYER_ENTERING_WORLD")
	UpgradeList:RegisterEvent("QUEST_DETAIL")
	UpgradeList:RegisterEvent("QUEST_COMPLETE")
	UpgradeList:RegisterEvent("CHAT_MSG_RAID")
	UpgradeList:RegisterEvent("CHAT_MSG_RAID_LEADER")
	UpgradeList:RegisterEvent("CHAT_MSG_RAID_WARNING")
	if GetNumTalentGroups("player") == 1 then
		UpgradeList:RegisterEvent("ACHIEVEMENT_EARNED")
	end
	
	--Hooks
	UpgradeList:SecureHook("ChatEdit_InsertLink") --Happens when a player shift-clicks an item
	
	UpgradeList:DrawUI()
	
end